;+
; Project     :	STEREO - SSC
;
; Name        :	SSC_MAKE_SECCHI_LINKS
;
; Purpose     :	Creates alternate Y/M/D SECCHI directory tree
;
; Category    :	STEREO, SSC
;
; Explanation :	Uses logical links to create an alternate SECCHI directory
;               structure where the date directories are converted into
;               separate year month and day directories.  For example,
;               "20110417" becomes "2011/04/17".  The original tree is
;               $SECCHI_LZ/L0, and the alternate tree is $SECCHI_LZ/L0_YMD.
;
; Syntax      :	SSC_MAKE_SECCHI_LINKS
;
; Examples    :	SSC_MAKE_SECCHI_LINKS, NDAYS=90
;
; Inputs      :	None
;
; Opt. Inputs :	None
;
; Outputs     :	The relevant directories and logical links are created under
;               $SECCHI_LZ/L0_YMD.
;
; Opt. Outputs:	None
;
; Env. vars.  : SECCHI_LZ = Top of the SECCHI data tree.
;
; Keywords    :	NDAYS   = Number of days to search backwards in time.  The
;                         default is 20.
;
;               FULL    = If set, then search over the full mission duration.
;
; Calls       :	GET_UTC, ANYTIM2UTC, CONCAT_DIR, FILE_EXIST, MK_DIR
;
; Common      :	None
;
; Restrictions:	None
;
; Side effects:	None
;
; Prev. Hist. :	None
;
; History     :	Version 1, William Thompson, GSFC, 01-Dec-2020
;
; Contact     :	WTHOMPSON
;-
;
pro ssc_make_secchi_links, ndays=ndays, full=full
;
;  Define the top directories.
;
secchi_lz = getenv('SECCHI_LZ')
secchi_l0 = concat_dir(secchi_lz, 'L0')
secchi_l0_ymd = concat_dir(secchi_lz, 'L0_YMD')
;
;  Form the date range.
;
get_utc, utc
mjd1 = utc.mjd
if n_elements(ndays) eq 0 then ndays = 20
mjd0 = mjd1 - ndays
if keyword_set(full) then begin
   utc = anytim2utc('2006-Oct-25')
   mjd0 = utc.mjd
endif
;
;  Step through the spacecraft.
;
sc = ['a', 'b']
for isc = 0,n_elements(sc)-1 do begin
   in_path0 = concat_dir(secchi_l0, sc[isc])
   outpath0 = concat_dir(secchi_l0_ymd, sc[isc])
   if file_exist(in_path0) and (not file_exist(outpath0)) then mk_dir, outpath0
;
;  Step through the data types
;
   type = ['cal','img','seq']
   for itype = 0,n_elements(type)-1 do begin
      in_path1 = concat_dir(in_path0, type[itype])
      outpath1 = concat_dir(outpath0, type[itype])
      if file_exist(in_path1) and (not file_exist(outpath1)) then $
         mk_dir, outpath1
;
;  Step through the telescopes.
;
      tel = ['cor1','cor2','euvi','hi_1','hi_2']
      for itel = 0,n_elements(tel)-1 do begin
         in_path2 = concat_dir(in_path1, tel[itel])
         outpath2 = concat_dir(outpath1, tel[itel])
         if file_exist(in_path2) then begin
            if not file_exist(outpath2) then mk_dir, outpath2
            print, 'Checking ' + in_path2
;
;  Step through the dates.
;
            for mjd = mjd0,mjd1 do begin
               utc.mjd = mjd
               ext = anytim2utc(utc, /external)
               year  = string(ext.year,  format='(I4.4)')
               month = string(ext.month, format='(I2.2)')
               day   = string(ext.day,   format='(I2.2)')
;
               in_path3 = concat_dir(in_path2, year+month+day)
               if file_exist(in_path3) then begin
                  outpath3 = concat_dir(outpath2, year)
                  if not file_exist(outpath3) then mk_dir, outpath3
                  outpath3 = concat_dir(outpath3, month)
                  if not file_exist(outpath3) then mk_dir, outpath3
                  outpath3 = concat_dir(outpath3, day)
                  if not file_exist(outpath3) then begin
                     file_link, in_path3, outpath3
                     print, 'Created ' + outpath3
                  endif         ;outpath3 does not exist
               endif            ;in_path3 exist
            endfor              ;imjd
         endif                  ;in_path2 exists
      endfor                    ;itel
   endfor                       ;itype
endfor                          ;isc
;
end
