;+
; Project     :	STEREO - SSC
;
; Name        :	SSC_UPDATE_HI
;
; Purpose     :	Ingest HI images from RAL archive
;
; Category    :	STEREO, SSC
;
; Explanation :	Ingests HI images from RAL archives, and makes secondary copy
;               on GSFC computers.
;
; Syntax      :	SSC_UPDATE_HI
;
; Examples    :	SSC_UPDATE_HI, RECENT=90
;
; Inputs      :	None
;
; Opt. Inputs :	None
;
; Outputs     :	Generates a log file in the directory logdir.
;
; Opt. Outputs:	None
;
; Env. vars.  : logdir  = Directory to write log files.
;               passfile= Password file used by rsync
;               destdir = Destination directory on SSC archive
;               b21dir  = Destination directory on secondary archive
;               remotedir = Remote directory on RAL archive
;
; Keywords    :	RECENT  = Time span to examine for most recent data.
;                         Otherwise, all dates from 2006-12-01 onward are
;                         processed.
;
;               LASTDATE = Last date to process.  Otherwise, all dates up to
;                          the present are processed.
;
; Calls       :	ANYTIM2CAL, CONCAT_DIR, ANYTIM2UTC, 
;
; Common      :	None
;
; Restrictions:	None
;
; Side effects:	None
;
; Prev. Hist. :	Earlier versions not in SSW
;
; History     :	Version 2, William Thompson, GSFC, 20-Apr-2018
;                       Include secondary archive
;               Version 3, WTT, GSFC, 23-Jun-2021, add timeout
;
; Contact     :	WTHOMPSON
;-
;
pro ssc_update_hi, recent=recent, lastdate=lastdate
;
;  Get the environment variables.
;
logdir     = getenv('logdir')
passfile   = getenv('passfile')
destdir    = getenv('destdir')
b21dir0    = getenv('b21dir')
remotedir0 = getenv('remotedir')
;
;  Get today's date, and form the name of the log file.
;
get_utc, utc
date = anytim2cal(utc, form=8, /date) + '_' + anytim2cal(utc, form=8, /time)
logfile = concat_dir(logdir, 'hi_rsync_' + date + '.log')
mjd1 = utc.mjd
;
;  If the keyword LASTDATE was passed, then only process up to that date.
;
if n_elements(lastdate) eq 1 then begin
   utc = anytim2utc(lastdate)
   mjd1 = utc.mjd
endif
;
;  If the RECENT keyword was passed, then use it to determine the start date to
;  process.  Otherwise, use 2006-12-01.
;
if n_elements(recent) eq 1 then mjd0 = mjd1 - recent else $
  mjd0 = (anytim2utc('2006-12-01')).mjd
help, mjd0, mjd1
;
;  Set up the types of data, the spacecraft, and the telescopes.
;
type = ['L2', 'L2_1_25', 'L2_11_25']
sc = ['a', 'b'] + '/img'
tel = ['hi_1', 'hi_2']
;
;  Step through the types of data, and start forming the directory trees.
;
for itype = 0,n_elements(type)-1 do begin
    remotedir1 = concat_dir(remotedir0, type[itype])
    localdir1  = concat_dir(destdir,    type[itype])
    b21dir1    = concat_dir(b21dir0,    type[itype])
;
;  Step through the spacecraft.
;
    for isc = 0,n_elements(sc)-1 do begin
        remotedir2 = concat_dir(remotedir1, sc[isc])
        localdir2  = concat_dir(localdir1,  sc[isc])
        b21dir2    = concat_dir(b21dir1,    sc[isc])
;
;  Step through the telescopes.
;
        for itel = 0,n_elements(tel)-1 do begin
            remotedir3 = concat_dir(remotedir2, tel[itel])
            localdir3  = concat_dir(localdir2,  tel[itel])
            b21dir     = concat_dir(b21dir2,    tel[itel])
;
;  Step through the dates.
;
            for mjd = mjd0,mjd1 do begin
                utc.mjd = mjd
                date = anytim2cal(utc, form=8, /date)
                remotedir = concat_dir(remotedir3, date)
                localdir  = concat_dir(localdir3, date)
;
;  Form the command to copy over the data from RAL.
;
                command = 'rsync -aWuv --password-file=' + $
                          passfile + ' --timeout=20 ' + remotedir + '/ ' + $
                          localdir + '/'
                if logfile ne '' then begin
                    command = command + ' >> ' + logfile
                    spawn, 'echo ' + remotedir + ' >> ' + logfile
                endif
		print, command
                spawn, command
;
;  Form the command to copy it over to the secondary archive.
;                
                command = 'rsync -aWuv --rsync-path=/usr/bin/rsync ' + $
                          '-e /usr/bin/ssh ' +localdir + ' ' + $
                          b21dir + '/'
                print, command
                spawn, command
            endfor              ;mjd
        endfor                  ;itel
    endfor                      ;isc
endfor                          ;itype
;
end
