;+
; Project     :	STEREO - SSC
;
; Name        :	SSC_BEACON_REFRESH_FILES
;
; Purpose     :	Refresh the STEREO beacon telemetry files
;
; Category    :	STEREO, Telemetry
;
; Explanation : This procedure is called by SSC_BEACON_READER to refresh the
;               STEREO beacon telemetry files for the IMPACT, PLASTIC, and
;               SWAVES instruments.  The procedure SSC_BEACON_OPEN_NEXT is
;               called for any file which has reached its end-of-file, to see
;               if a newer file is available.  Files are refreshed for both the
;               Ahead and Behind spacecraft.
;
; Syntax      :	SSC_BEACON_REFRESH_FILES, IMPACTLUN, PLASTICLUN, SWAVESLUN
;
; Examples    :	See SSC_BEACON_READER.
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	IMPACTLUN, PLASTICLUN, SWAVESLUN
;                       Each is a two-element array containing the logical unit
;                       numbers of the Ahead and Behind files for each
;                       instrument.
;
; Opt. Outputs:	SECCHILUN = Returned only if /HEADER_ONLY is set.
;
; Keywords    :	NOWAIT  = Normally, if all the files are still at the
;                         end-of-file mark at the end of this procedure, a
;                         small wait is put in to control CPU usage.  If
;                         /NOWAIT is passed, then this wait is bypassed.
;
;               DONE    = Returns 1 (true) if all the files are at their
;                         end-of-file mark.
;
;               HEADER_ONLY = If set, then open the header-only beacon
;                             ".ptp.hdr" files.
;
; Calls       :	SSC_BEACON_OPEN_NEXT
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 08-Feb-2006
;               Version 2, 08-Jul-2009, WTT, added /HEADER_ONLY keyword
;
; Contact     :	WTHOMPSON
;-
;
pro ssc_beacon_refresh_files, impactlun, plasticlun, swaveslun, secchilun, $
                              done=done, nowait=nowait, header_only=header_only
on_error, 2
;
n_done = 0
for isc=0,1 do begin
    if eof(impactlun[isc]) then ssc_beacon_open_next, impactlun, isc, $
      header_only=header_only
    if eof(impactlun[isc]) then n_done = n_done + 1
;
    if eof(plasticlun[isc]) then ssc_beacon_open_next, plasticlun, isc, $
      header_only=header_only
    if eof(plasticlun[isc]) then n_done = n_done + 1
;
    if eof(swaveslun[isc]) then ssc_beacon_open_next, swaveslun, isc, $
      header_only=header_only
    if eof(swaveslun[isc]) then n_done = n_done + 1
;
    if keyword_set(header_only) then begin
        if eof(secchilun[isc]) then ssc_beacon_open_next, secchilun, isc, $
          header_only=header_only
        if eof(secchilun[isc]) then n_done = n_done + 1
    endif
endfor
;
n_done_max = 6
if keyword_set(header_only) then n_done_max = 8
done = (n_done eq n_done_max)
if done and not keyword_set(nowait) then wait, 10
return
end
