;+
; Project     :	STEREO - SSC
;
; Name        :	SSC_EUVI_SYNOPTIC
;
; Purpose     :	Form synoptic images from beacon data.
;
; Category    :	STEREO, SECCHI, Coordinates
;
; Explanation :	This routine forms synoptic maps from SECCHI EUVI beacon
;               images.  The most recent matched pair of EUVI images are read
;               in, converted to heliographic coordinates, and combined.
;               Matching SDO/AIA data are also used when available.
;
; Syntax      :	SSC_EUVI_SYNOPTIC  [, MAP, ORIGIN ]
;
; Examples    :	SSC_EUVI_SYNOPTIC
;               IMAGE = TVREAD(RED,GREEN,BLUE)
;               WRITE_GIF, 'euvi_synoptic.gif', IMAGE, RED, GREEN, BLUE
;
; Inputs      :	None required
;
; Opt. Inputs :	None.
;
; Outputs     :	The output of the program is a plot of the synoptic map.
;
; Opt. Outputs:	MAP     = Synoptic map, byte-scaled for display
;
;               ORIGIN  = Coordinates of lower-left corner, for use with
;                         plot_image.
;
; Keywords    :	SCALE   = Map scale, in degrees.  The default is 1.
;
;               GRID    = If set, then a grid is overplotted on the image at 60
;                         degree intervals in longitude, and 30 degree
;                         intervals in latitude.
;
;               COLOR   = If set, then use SECCHI_PREP to load the color table.
;
;               NPIXELS = Two-element array containing the number of pixels in
;                         the longitude and latitude directions.  When this
;                         option is passed, the SCALE keyword is ignored.
;                         Also, the resulting image is displayed without any
;                         labeling.  Used for generating images to be used in
;                         spherical displays.
;
;               DATE_OBS= Returns the observation date to the calling routine.
;
;               NOAIA   = If set, then don't process the AIA image.  This
;                         option is intended mainly for imaging the far side of
;                         the Sun.
;
;               REQUIRE_AIA= If set, then require that a corresponding AIA
;                         image be found.  Ignored for dates prior to
;                         2010-05-13.
;
;               ERRMSG  = If defined, then return any error messages in this
;                         variable.
;
;               Additional keywords are passed to the routines
;               SSC_GATHER_EUVI_SYNOPTIC and SSC_FORM_EUVI_SYNOPTIC.
;
; Calls       :	GET_UTC, SCC_READ_SUMMARY, UTC2TAI, SECCHI_PREP, FITSHEAD2WCS,
;               WCS_CONVERT_TO_COORD, WCS_GET_PIXEL, INTERPOLATE,
;               WHERE_MISSING, SCC_BYTSCL, PLOT_IMAGE, SSC_GET_NEAREST_AIA,
;               WCS_CONVERT_DIFF_ROT, WCS_RSUN, SSC_GET_AIA_304_FACTOR
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 8-Apr-2009, William Thompson, GSFC
;               Version 2, 9-Apr-2009, WTT, filter out images with NMISS>0
;               Version 3, 7-May-2009, WTT, include observation date on plot
;               Version 4, 29-Jul-2009, WTT, added keyword NPIXELS
;               Version 5, 12-Nov-2009, WTT, added keyword CEA
;               Version 6, 13-Nov-2009, WTT, added keyword GRID
;               Version 7, 01-Feb-2010, WTT, added keyword BACKGRID
;               Version 8, 22-Jun-2010, WTT, include SDO/AIA data
;               Version 9, 25-Jun-2010, WTT, more robust logic for SDO/AIA
;               Version 10, 27-Sep-2010, WTT, corrected bug when at 180 degrees
;               Version 11, 30-Sep-2010, WTT, use CACHE_AIA for current data
;               Version 12, 05-Jan-2011, WTT, added keyword FILL_MISSING
;                       Improved AIA 304 scaling.  Combine 284 with AIA 211
;               Version 13, 06-Jan-2011, WTT, pass through VSO keywords
;               Version 14, 10-Jan-2011, WTT, added keyword FILENAME
;               Version 15, 13-Jan-2011, WTT, added keyword NOAIA
;               Version 16, 03-Feb-2011, WTT, added keyword NOSEARCH
;               Version 17, 07-Feb-2011, WTT, fixed NOSEARCH bug when no AIA data
;               Version 18, 09-Feb-2011, WTT, call SSC_GET_AIA_304_FACTOR
;               Version 19, 09-Apr-2012, WTT, added keyword REQUIRE_AIA
;               Version 20, 28-Jun-2012, WTT, don't look for AIA before 2010-05-13
;               Version 21, 21-Apr-2014, WTT, added keyword SINGLE_SC
;               Version 22, 24-Jul-2014, WTT, rewrote to call
;                       SSC_GATHER_EUVI_SYNOPTIC and SSC_FORM_EUVI_SYNOPTIC
;                       Moved many keywords to those routines.
;               Version 23, 28-Jul-2014, WTT, corrected bug with COLOR keyword
;
; Contact     :	WTHOMPSON
;-
;
pro ssc_euvi_synoptic, map, origin, scale=scale, npixels=npixels, color=color, $
                       errmsg=errmsg, date_obs=date_obs, cea=cea, grid=grid, $
                       noaia=noaia, require_aia=require_aia, _extra=_extra
;
on_error, 2
;
;  Define some defaults.
;
if n_elements(scale) ne 1 then scale = 1
radeg = 180.d0 / !dpi
;
;  Call SSC_GATHER_EUVI_SYNOPTIC to read in the images.
;
message = ''
if keyword_set(noaia) then $
    ssc_gather_euvi_synoptic, ha, a, hb, b, color=color, errmsg=message, $
                              _extra=_extra else $
    ssc_gather_euvi_synoptic, ha, a, hb, b, hc, c, color=color, $
                              errmsg=message, _extra=_extra
if message ne '' then goto, handle_error
;
if n_elements(ha) eq 1 then dt_obs = ha.date_obs else dt_obs = hb.date_obs
if keyword_set(require_aia) and (dt_obs gt '2010-05-13') and $
  (n_elements(hc) eq 0) then begin
    message = 'AIA image not found'
    goto, handle_error
endif
;
;  Call SSC_FORM_EUVI_SYNOPTIC to form the map.
;
ssc_form_euvi_synoptic, map, origin, ha, a, hb, b, hc, c, scale=scale, $
                        npixels=npixels, errmsg=message, date_obs=date_obs, $
                        cea=cea, _extra=_extra
if message ne '' then goto, handle_error
;
;  If NPIXELS is passed, then simply display the image
;
if n_elements(npixels) eq 2 then begin
    tv, map
;
;  Overplot the grid, if requested.
;
    if keyword_set(grid) then begin
        xtickv = npixels[0] * (indgen(5) + 1.) / 6
        for i=0,4 do plots, replicate(xtickv[i],2), [0,npixels[1]-1], /device
        if keyword_set(cea) then begin
            theta = (-60+30*indgen(5)) / radeg
            ytickv = (sin(theta) + 1) * (npixels[1]-1) / 2
        end else ytickv = npixels[1] * (indgen(5) + 1.) / 6
        for i=0,4 do plots, [0,npixels[0]-1], replicate(ytickv[i],2), /device
    endif
;
;  Otherwise, plot the map.
;
end else begin
    xtickv = origin[0] + 30*indgen(13)
    ytickv = origin[1] + 30*indgen(7)
    ytickname = ntrim(round(ytickv))
    if keyword_set(cea) then ytickv = 90 * sin(ytickv/radeg)
    plot_image, map, origin=origin, scale=scale, /noscale, ticklen=-0.01, $
      xtitle = 'Longitude', ytitle='Latitude', $
      xticks=12, yticks=6, xtickv=xtickv, ytickv=ytickv, ytickname=ytickname, $
      _extra=_extra
;
;  Overplot the grid, if requested.
;
    if keyword_set(grid) then begin
        for i=2,10,2 do oplot, replicate(xtickv[i],2), !y.crange
        for i=1,n_elements(ytickv)-2 do oplot, !x.crange, $
          replicate(ytickv[i],2)
    endif
;
    date = anytim2utc(date_obs, /ecs, /truncate)
    xyouts, !d.x_size/2, !d.y_ch_size/2, alignment=0.5, /device, $
      'Observation date: ' + date, _extra=_extra
endelse
return
;
;  Error handling point.
;
handle_error:
if n_elements(errmsg) eq 0 then message, message else errmsg=message
;
end
