;+
; Project     :	STEREO - SSC
;
; Name        :	SSC_GET_AIA_304_FACTOR()
;
; Purpose     :	Returns rough conversion factor for AIA 304 images
;
; Category    :	STEREO, SECCHI, Coordinates
;
; Explanation :	This routine returns a time-dependent conversion factor between
;               the SDO/AIA 304 images and the STEREO/EUVI 304 images for use
;               within the SSC_EUVI_SYNOPTIC program.  This conversion factor
;               is intended *only* for this purpose, and should not be
;               interpreted as an actual cross-calibration factor.
;
;               The first time this routine is called, the file
;               ssc_aia_304_factor.dat in the IDL path is read in and stored in
;               the common block.  If this file is not found, the program
;               defaults to the conversion factor 15.
;
; Syntax      :	Factor = SSC_GET_AIA_304_FACTOR( DATE )
;
; Examples    :	See SSC_EUVI_SYNOPTIC
;
; Inputs      :	DATE = The observation date.
;
; Opt. Inputs :	None.
;
; Outputs     :	The output of the program is the conversion factor.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	FIND_WITH_DEF, UTC2TAI, ANYTIM2TAI
;
; Common      :	Common block SSC_GET_AIA_304_FACTOR stores the factors.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 9-Feb-2011, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
function ssc_get_aia_304_factor, date
;
common ssc_get_aia_304_factor, tai, factor
;
;  If this is the first time the routine is called, find the data file and read
;  it into the common block.  If the data file cannot be found, then return the
;  default value of 15.
;
if n_elements(tai) eq 0 then begin
    filename = find_with_def('ssc_aia_304_factor.dat', !path)
    if filename eq '' then return, 15 else begin
        openr, unit, filename, /get_lun
        while not eof(unit) do begin
            line = 'string'
            readf, unit, line
            words = strsplit(line, /extract)
            if n_elements(tai) eq 0 then begin
                tai = utc2tai(words[0])
                factor = float(words[1])
            end else begin
                tai = [tai, utc2tai(words[0])]
                factor = [factor, float(words[1])]
            endelse
        endwhile
        free_lun, unit
    endelse
endif
;
;  Return the nearest factor found.
;
tai0 = anytim2tai(date)
diff = abs(tai-tai0)
w = where(diff eq min(diff))
return, factor[w[0]]
end
