;+
; Project     :	STEREO - SSC
;
; Name        :	SSC_INSITU_BEACON_CARR
;
; Purpose     :	Plot STEREO solar window data as function of rotation.
;
; Category    :	STEREO, IMPACT, PLASTIC
;
; Explanation :	Plots STEREO in situ solar window data as a function of
;               Carrington rotation number, for display on the web.  The
;               beginning and end dates are calculated for each Carrington
;               rotation as seen from Earth.
;
; Syntax      :	SSC_INSITU_BEACON_CARR, CARR_ROT
;
; Examples    :	SSC_INSITU_BEACON_CARR, 2090, TIMERANGE=TIMERANGE, SMOOTH=3600
;
; Inputs      :	CARR_ROT = Carrington rotation number.  The data are plotted
;                          over the times corresponding to CARR_ROT to
;                          CARR_ROT+1.  Can be fractional, e.g. 2090.5.  Must
;                          be between 2050 and the current value.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result is a plot of the solar wind data to the current
;               graphics device.
;
; Opt. Outputs:	None.
;
; Keywords    :	SMOOTH    = The width in seconds to use for smoothing the data.
;
;               TIMERANGE = Returns the time range used for plotting the data.
;
;               TICK_UNIT = Number of seconds between tick marks.  Default is
;                           six days.
;
; Calls       :	GET_UTC, GET_STEREO_CARR_ROT, GET_STEREO_CARR_DATE, ANYTIM2CAL,
;               SSC_BEACON_PATH, CONCAT_DIR, SSC_INSITU_BEACON_PANELS,
;               SSC_UPDATE_PLASTIC_CDF
;
; Common      :	None, but underlying routines use ssc_insitu_beacon.cmn.
;
; Env. Vars.  : Underlying routines use STEREO_BEACON_DATA
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 01-Jul-2010, William Thompson, GSFC
;               Version 2, 17-Oct-2023, WTT, add call to SSC_UPDATE_PLASTIC_CDF
;
; Contact     :	WTHOMPSON
;-
;
pro ssc_insitu_beacon_carr, carr_rot, smooth=smooth, timerange=timerange, $
                            tick_unit=tick_unit
;
on_error, 2
;
if n_elements(tick_unit) eq 0 then tick_unit = 6.*86400.d0      ;Six days
;
;  Check the Carrington rotation number.
;
if n_elements(carr_rot) ne 1 then message, 'CARR_ROT must be a scalar'
get_utc, today
max_carr_rot = get_stereo_carr_rot(today, 'Earth')
if (carr_rot lt 2050) or (carr_rot gt max_carr_rot) then message, $
  'Carrington rotation number out of range'
;
;  Get the time ranges for Earth.
;
timerange = [get_stereo_carr_date(carr_rot,   'earth'), $
             get_stereo_carr_date(carr_rot+1, 'earth')]
;
;  Find all the Ahead files.
;
mjd0 = timerange[0].mjd
mjd1 = timerange[1].mjd
for mjd = mjd0,mjd1 do begin
    date = {mjd: mjd, time: 0L}
    s_date = anytim2cal(date, form=8, /date)
;
    path = ssc_beacon_path('STA', 'impact', date)
    test = concat_dir(path, 'STA_LB_IMPACT_'  + s_date + '*.cdf')
    files = file_search(test)
    if mjd eq mjd0 then impact_a = files else impact_a = [impact_a, files]
;
    path = ssc_beacon_path('STA', 'plastic', date)
    test = concat_dir(path, 'STA_LB_PLASTIC_'  + s_date + '*.cdf')
    files = file_search(test)
    if mjd eq mjd0 then plastic_a = files else plastic_a = [plastic_a,files]
endfor
;
;  Find all the Behind files.
;
mjd0 = timerange[0].mjd
mjd1 = timerange[1].mjd
for mjd = mjd0,mjd1 do begin
    date = {mjd: mjd, time: 0L}
    s_date = anytim2cal(date, form=8, /date)
;
    path = ssc_beacon_path('STB', 'impact', date)
    test = concat_dir(path, 'STB_LB_IMPACT_'  + s_date + '*.cdf')
    files = file_search(test)
    if mjd eq mjd0 then impact_b = files else impact_b = [impact_b, files]
;
    path = ssc_beacon_path('STB', 'plastic', date)
    test = concat_dir(path, 'STB_LB_PLASTIC_'  + s_date + '*.cdf')
    files = file_search(test)
    if mjd eq mjd0 then plastic_b = files else plastic_b = [plastic_b,files]
endfor
;
;  Replace the PLASTIC beacon files with science files if found.
;
    ssc_update_plastic_cdf, plastic_a
    ssc_update_plastic_cdf, plastic_b
;
; Set the plot title.
;
title = 'IMPACT/PLASTIC  rotation ' + ntrim(carr_rot)
;
;  Do the plot.
;
ssc_insitu_beacon_panels, impact_a, impact_b, plastic_a, plastic_b, $
  timerange=timerange, tick_unit=tick_unit, title=title, smooth=smooth
;
return
end
