;+
; Project     :	STEREO - SSC
;
; Name        :	SSC_MIRROR_DSCOVR_FILES
;
; Purpose     :	Mirror and uncompress DSCOVR solar wind files
;
; Category    :	Space weather
;
; Explanation :	This routine copies over the DSCOVR 1-minute average plasma and
;               magnetic field files from the NOAA website, decompresses them,
;               and arranges them in a directory tree under $SSW_DSCOVR_DATA.
;
; Syntax      :	SSC_MIRROR_DSCOVR_FILES  [, DATE ]
;
; Examples    :	SSC_MIRROR_DSCOVR_FILES                 ;Processes last 10 days
;               SSC_MIRROR_DSCOVR_FILES, '2016-07-26'
;
; Inputs      :	None required
;
; Opt. Inputs :	None
;
; Outputs     :	New files are installed in $SSW_DSCOVR_DATA
;
; Opt. Outputs:	None
;
; Keywords    :	None
;
; Calls       :	GET_UTC, ANYTIM2UTC, CONCAT_DIR, MK_DIR
;
; Common      :	None
;
; Restrictions:	Must have write privilege to $SSW_DSCOVR_DATA
;
; Side effects:	None
;
; Prev. Hist. :	None
;
; History     :	Version 1, William Thompson, GSFC, 12-Aug-2016
;
; Contact     :	WTHOMPSON
;-
;
pro ssc_mirror_dscovr_files, date
;
;  If DATE was not passed, then call this routine re-iteratively for the last
;  10 days.
;
if n_params() eq 0 then begin
    get_utc, utc
    for i=0,9 do begin
        utc.mjd = utc.mjd - 1
        ssc_mirror_dscovr_files, utc
    endfor
    return
endif
;
;  Form the remote and output locations.  If necessary, create the output
;  directories.
;
ext = anytim2utc(date, /external)
;
year = string(ext.year,format='(I4.4)')
outdir = concat_dir(getenv('SSW_DSCOVR_DATA'), year)
if ~file_exist(outdir) then mk_dir, outdir
;
month = year + '/' + string(ext.month,format='(I2.2)')
outdir = concat_dir(getenv('SSW_DSCOVR_DATA'), month)
if ~file_exist(outdir) then mk_dir, outdir
;
url = 'http://www.ngdc.noaa.gov/dscovr/data/' + month
;
;  Get a list of all the files at the remote location.
;
spawn, 'wget ' + url + ' -O -', output
;
;  Form the beginnings of the file names to be looked for.
;
sdate = anytim2cal(ext, form=8, /date)
filetype = 'oe_' + ['f','m'] + '1m_dscovr_s' + sdate
;
;  Step through the output, and look for the requested files.
;
for itype = 0,n_elements(filetype)-1 do begin
    for i=0,n_elements(output)-1 do begin
        j0 = strpos(output[i], filetype[itype])
;
;  If the beginning of the file is found, then extract out the complete
;  (uncompressed) filename.
;
        if j0 gt 0 then begin
            j1 = strpos(output[i], '.nc.gz')
            filename = strmid(output[i], j0, j1-j0+3)
;
;  If the file hasn't already been copied over, then copy it to the
;  appropriate directory, and decompress it.
;
            outfile = concat_dir(outdir, filename)
            if ~file_exist(outfile) then begin
                outfile = outfile + '.gz'
                spawn, 'wget ' + url + '/' + filename + '.gz -O ' + outfile
                spawn, 'gunzip ' + outfile
            endif
        endif
    endfor
endfor
;
end
