;+
; Project     :	STEREO - SSC
;
; Name        :	SSC_READ_WIND_MAG
;
; Purpose     :	Read WIND mag data from local file
;
; Category    :	STEREO, Telemetry
;
; Explanation :	This routine is called from SSC_GET_WINDDATA to read in a local
;               copy of the WIND mag data file in versions of IDL prior to
;               8.3 which don't support https.
;
; Syntax      :	SSC_READ_WIND_MAG, T0, T1, MAG
;
; Examples    :	See SSC_GET_WINDDATA
;
; Inputs      :	T0, T1  = Time range
;
; Opt. Inputs :	None
;
; Outputs     :	MAG     = Structure array containing mag data
;
; Opt. Outputs:	None
;
; Env. Vars.  : WINDDATA = Directory containing WIND 7-day data
;
; Keywords    :	None
;
; Calls       :	CONCAT_DIR, FILE_EXIST, DATA_CHK, SSW_TIME2EPOCH
;
; Common      :	None
;
; Restrictions:	None
;
; Side effects:	None
;
; Prev. Hist. :	None
;
; History     :	Version 1, 08-Aug-2019, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
pro ssc_read_wind_mag, t0, t1, mag
;
mag = {time_tag: '', b_gsm: fltarr(3), lon_gsm: 0., lat_gsm: 0., bt: 0.}
mag0 = mag
;
filename = concat_dir(getenv('WINDDATA'), 'mag-7-day.json')
if not file_exist(filename) then return
if file_test(filename, /zero_length) then return
;
catch, error_status
if error_status ne 0 then begin
    catch, /cancel
    mag = mag0
    return
endif
;
openr, unit, filename, /get_lun
json = 'String'
readf, unit, json
free_lun, unit
;
list = json_parse(json, /tostruct)
array = list.toarray(missing='NaN')
data = float(array[1:*,*])
count = data_chk(data,/nx)      ;Number of returned elements
;
mag = replicate(mag, count)
mag.time_tag = array[1:*,0]
mag.b_gsm    = transpose(data[*,1:3])
mag.lon_gsm  = data[*,4]
mag.lat_gsm  = data[*,5]
mag.bt       = data[*,6]
;
epoch = ssw_time2epoch(mag.time_tag, t0, t1)
ssok = where(epoch eq 0, count)
if count gt 0 then mag = mag[ssok] else mag = mag0
;
end
