;+
; Project     :	STEREO - SSC
;
; Name        :	SSC_READ_WIND_PLASMA
;
; Purpose     :	Read WIND plasma data from local file
;
; Category    :	STEREO, Telemetry
;
; Explanation :	This routine is called from SSC_GET_WINDDATA to read in a local
;               copy of the WIND plasma data file in versions of IDL prior to
;               8.3 which don't support https.
;
; Syntax      :	SSC_READ_WIND_PLASMA, T0, T1, PLASMA
;
; Examples    :	See SSC_GET_WINDDATA
;
; Inputs      :	T0, T1  = Time range
;
; Opt. Inputs :	None
;
; Outputs     :	PLASMA  = Structure array containing plasma data
;
; Opt. Outputs:	None
;
; Env. Vars.  : WINDDATA = Directory containing WIND 7-day data
;
; Keywords    :	None
;
; Calls       :	CONCAT_DIR, FILE_EXIST, DATA_CHK, SSW_TIME2EPOCH
;
; Common      :	None
;
; Restrictions:	None
;
; Side effects:	None
;
; Prev. Hist. :	None
;
; History     :	Version 1, 08-Aug-2019, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
pro ssc_read_wind_plasma, t0, t1, plasma
;
plasma = {time_tag: '', density: 0., speed: 0., temperature: 0.}
plasma0 = plasma
filename = concat_dir(getenv('WINDDATA'), 'plasma-7-day.json')
if not file_exist(filename) then return
if file_test(filename, /zero_length) then return
;
catch, error_status
if error_status ne 0 then begin
    catch, /cancel
    plasma = plasma0
    return
endif
;
openr, unit, filename, /get_lun
json = 'String'
readf, unit, json
free_lun, unit
;
list = json_parse(json, /tostruct)
array = list.toarray(missing='NaN')
data = float(array[1:*,*])
count = data_chk(data,/nx)      ;Number of returned elements
;
plasma = replicate(plasma, count)
plasma.time_tag    = array[1:*,0]
plasma.density     = data[*,1]
plasma.speed       = data[*,2]
plasma.temperature = data[*,3]
;
epoch = ssw_time2epoch(plasma.time_tag, t0, t1)
ssok = where(epoch eq 0, count)
if count gt 0 then plasma = plasma[ssok] else plasma = plasma0
;
end
