;+
; Project     :	STEREO - SSC
;
; Name        :	SSC_SWAVES_SWX_DAILY
;
; Purpose     :	Plot the SWAVES beacon data for a given date
;
; Category    :	STEREO, Telemetry
;
; Explanation :	Reads in SWAVES beacon IDL save files for a given date, and
;               plots the results to a GIF file in the browse directory tree.
;
; Syntax      :	SSC_SWAVES_SWX_DAILY, DATE
;
; Examples    :	SSC_SWAVES_SWX_DAILY, '2021/09/03'
;
; Inputs      :	DATE    = Date to process
;
; Opt. Inputs :	None
;
; Outputs     :	Writes beacon_swaves.gif
;
; Opt. Outputs:	None
;
; Keywords    :	None
;
; Calls       :	ANYTIM2UTC, ANYTIM2CAL, CONCAT_DIR, FILE_EXIST, UTC2TAI, TVLCT,
;               CHECK_INT_TIME, UTPLOT_IMAGE, TVREAD
;
; Common      :	None
;
; Restrictions:	None
;
; Side effects:	None
;
; Prev. Hist. :	Based on SSC_SWAVES_SWX_PLOT
;
; History     :	Version 1, 03-Sep-2021, William Thompson, GSFC
;               Version 2, 09-Sep-2021, WTT, handle missing date
;               Version 3, 08-Mar-2022, WTT, fix date axis problem
;               Version 4, 28-Jul-2023, WTT, only include A post conjunction
;
; Contact     :	WTHOMPSON
;-
;
pro ssc_swaves_swx_daily, date
;
message = ''
sdate = anytim2utc(date, /ecs, /date_only, errmsg=message)
if message ne '' then begin
   print, 'SSC_SWAVES_SWX_DAILY: ' + message
   return
endif
datedir = strmid(sdate, 0, 8)
caldate = anytim2cal(date, form=8, /date)
;
newy = indgen(318)
freqlo=0.125+findgen(318)*.05
hour=findgen(1800)/60.
;
stereo_beacon_data = getenv('STEREO_BEACON_DATA')
;
tai0 = utc2tai(caldate)
utc = anytim2utc(tai0+60*dindgen(1440))
speca = lonarr(318,1440)
specb = lonarr(318,1440)
;
file = concat_dir(stereo_beacon_data, 'ahead/swaves/' + datedir + $
                  'STA_LB_SWAVES_' + caldate + '.idlsave')
if file_exist(file) then begin
   restore, file
   index = fix((utc2tai(specdate)-tai0)/60)
   speca[*,index] = spectra
end
;
file = concat_dir(stereo_beacon_data, 'behind/swaves/' + datedir + $
                  'STB_LB_SWAVES_' + caldate + '.idlsave')
if file_exist(file) then begin
   restore, file
   index = fix((utc2tai(specdate)-tai0)/60)
   specb[*,index] = spectra
endif
;
speca = transpose(speca)
specb = transpose(specb)
array_a = fltarr(1440,318)
array_b = fltarr(1440,318)
;
p_title='STEREO WAVES beacon data for ' + sdate
; Load w-b color table
;
wb=255-indgen(256)
tvlct,wb,wb,wb
;
;  Select the Z buffer
;
dname = !d.name
set_plot,'z'
device, set_resolution=[800,600]
;
; Set plotting background to white, labelling to black (most color tables)
;
!p.background=0
!p.color=255
flow=0.125
fhigh=16.025
fspace=.050
funit='MHz'
linlog=1
yll='log'
;  Need to interpolate over missing data

for i = 0,1439 do begin
    lzer = where(speca(i,*) gt 0.)
    if(n_elements(lzer) lt 5) then goto,next_b
    v = speca(i,lzer)
    array_a(i,0:317)= interpol(v,lzer,newy)>0.
next_b:
    lzer = where(specb(i,*) gt 0.)
    if(n_elements(lzer) lt 5) then goto,next_time
    v = specb(i,lzer)
    array_b(i,0:317)= interpol(v,lzer,newy)>0.
next_time:
endfor
;
; background subtraction
;
for fr=0,317 do begin
    tot=total(array_a(*,fr))
    bkgnd=0.
    if  (tot le 0.) then goto,bback
    gtz=where(array_a(*,fr) gt 0.)
    bkgnd=min(array_a(gtz,fr))
    array_a(*,fr)=array_a(*,fr)-bkgnd
bback:
    tot=total(array_b(*,fr))
    bkgnd=0.
    if  (tot le 0.) then goto,nextb
    gtz=where(array_b(*,fr) gt 0.)
    bkgnd=min(array_b(gtz,fr))
    array_b(*,fr)=array_b(*,fr)-bkgnd
nextb:
endfor
plot_data:
scale1=0.0
scale2=10.0
nticks=10

erase

ncolors=(!d.table_size<256)-3
colors=indgen(ncolors)
bar=intarr(ncolors,5)
for j=0,4 do bar(*,j)=colors
contour,bar,position=[.2,.075,.8,.1],xrange=[scale1,scale2],xticks=nticks,$
xstyle=1,ystyle=4,font=0,$
xtitle='intensity (dB) relative to background',/nodata,/noerase
px=!x.window*!d.x_vsize
py=!y.window*!d.y_vsize
sx=px(1)-px(0)+1
sy=py(1)-py(0)+1
tv,poly_2d(byte(bar),[[0,0],[ncolors/sx,0]],[[0,5/sy],$
[0,0]],0,sx,sy),px(0),py(0)
xyouts,.5,.81,/normal,font=0,p_title,charsize=2.0,alignment=0.5
arrayab=bytscl(array_a,min=scale1,max=scale2,top=ncolors-1)
arraybb=bytscl(array_b,min=scale1,max=scale2,top=ncolors-1)
;
utc = anytim2utc(sdate)
utc = replicate(utc, 1440)
utc.time = utc.time + hour*3.6d6
check_int_time, utc
;
;  If post-conjuction, then only plot the Ahead data, in the normal
;  orientation.
;
if utc[0].mjd ge 57161 then begin
    utplot_image, arrayab, utc, alog10(freqlo), position=[.1,.2,.9,.78],$
                  xstyle=1,ystyle=5,/noscale,ytitle=funit,ticklen=-.01,$
                  /noerase,xtitle='Time of day',tick_unit=tick_unit
    axis, yaxis=0, ystyle=1, /ylog, ticklen=-0.01, ytitle=funit
    axis, yaxis=1, ystyle=1, /ylog, ticklen=-0.01, ytickformat='blankticks'
    red = ncolors
    linecolor, red, 'red'
    xyouts,.915,.48,'Ahead',orientation=270,/normal,alignment=0.5,color=red
;
;  Otherwise, plot both Ahead and Behind
;
end else begin
    utplot_image, reverse(arrayab,2), utc, -reverse(alog10(freqlo)), $
                  position=[.1,.5,.9,.78],tick_unit=tick_unit,$
                  xstyle=5,ystyle=5,/noscale,ytitle=funit,ticklen=-.01,$
                  /noerase,yaxis=1,xtitle=''
    !y.crange=-!y.crange
    axis, xaxis=1, xstyle=1, ticklen=-0.01, xtickformat='blankticks', xticks=24
    axis, yaxis=0, ystyle=1, /ylog, ticklen=-0.01, ytitle=funit
    axis, yaxis=1, ystyle=1, /ylog, ticklen=-0.01, ytickformat='blankticks'
    red = ncolors
    linecolor, red, 'red'
    xyouts,.915,.62,'Ahead',orientation=270,/normal,alignment=0.5,color=red
;
    utplot_image, arraybb, utc, alog10(freqlo), position=[.1,.2,.9,.48],$
                  xstyle=9,ystyle=5,/noscale,ytitle=funit,ticklen=-.01,$
                  /noerase,xtitle='Time of day',tick_unit=tick_unit
    axis, yaxis=0, ystyle=1, /ylog, ticklen=-0.01, ytitle=funit
    axis, yaxis=1, ystyle=1, /ylog, ticklen=-0.01, ytickformat='blankticks'
    blue = ncolors+1
    linecolor, blue, 'blue'
    xyouts,.915,.34,'Behind',orientation=270,/normal,alignment=0.5,color=blue
endelse
;
image = tvread(red, green, blue, /quiet)
filepath = getenv('STEREO_BROWSE')
if filepath ne '' then filepath = concat_dir(filepath, sdate)
filename = concat_dir(filepath, 'beacon_swaves.gif')
write_gif, filename, image, red, green, blue
set_plot, dname
print, 'Wrote ' + filename
;
end
