;+
; Project     :	STEREO - SWAVES
;
; Name        :	SSC_SWAVES_SWX_DECOM
;
; Purpose     :	Parse SWAVES space weather packets
;
; Category    :	STEREO, Telemetry
;
; Explanation :	Takes a SWAVES space weather packet read in with
;               READ_STEREO_PKT, and returns the spectrum and frequency arrays.
;
; Syntax      :	SSC_SWAVES_SWX_DECOM, PACKET, SPECTRUM_HF, FREQUENCY_HF
;
; Examples    :	See ssc_beacon_reader.pro
;
; Inputs      :	PACKET  = The space weather telemetry packet, as read in by
;                         READ_STEREO_PKT.
;
; Opt. Inputs :	None.
;
; Outputs     :	SPECTRUM_HF  = The radio spectrum.  This is an array of 318
;                              values.  The values are roughly in dBs.  There
;                              are 2 kinds of missing values in this array: -1
;                              indicates that the frequency value in question
;                              is not included in the current instrument parity
;                              selection.  A zero indicates that the frequency
;                              value in question was not sampled during the
;                              period.  The values 0 and -1 do not otherwise
;                              appear in the spectra.  When there is a CAL
;                              going on, the existing spectral values are
;                              replaced with zeroes.
;
; Opt. Outputs: FREQUENCY_HF = The frequency array, in Hz.  This never changes,
;                              and is only calculated if this parameter is
;                              passed.
;
; Keywords    :	None.
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	Only space weather packets are correctly handled.
;
; Side effects:	None.
;
; Prev. Hist. :	From SWAVES_SWX_DECOM by Keith Goetz
;
; History     :	Version 1, 27-Nov-2007, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;

pro SSC_SWAVES_SWX_decom, PACKET, Spectrum_HF, Frequency_HF

DAT = packet.dat

;
; most of the following scalar variables are actually unused here but are included
;	for reference and completeness
;

;;CheckSum		= long(DAT[0])
;;SubHead			= long(DAT[1])

;;TDS_MSB			= long(DAT[2])
;;TDS_LSB			= long(DAT[3])

Mode_SWX		= long(DAT[4])

;;Mode_LFRaMSB		= long(DAT[5])
;;Mode_LFRaLSB		= long(DAT[6])
;;Mode_LFRbMSB		= long(DAT[7])
;;Mode_LFRbLSB		= long(DAT[8])
;;Mode_LFRcMSB		= long(DAT[9])
;;Mode_LFRcLSB		= long(DAT[10])
;;Mode_HFR1MSB		= long(DAT[11])
;;Mode_HFR1LSB		= long(DAT[12])
;;Mode_HFR2MSB		= long(DAT[13])
;;Mode_HFR3LSB		= long(DAT[14])

;;Power_LFRa		= long(DAT[15])
;;Power_LFRb		= long(DAT[16])
;;Power_LFRc		= long(DAT[17])

;
; here's where we really get going
;

Power_HF		= long(DAT[18:176])			;raw SWX spectrum

Mode_SWX_CAL		= Mode_SWX AND 224L
Mode_SWX_CAL		= Mode_SWX_Cal/32L
Mode_SWX_Parity		= Mode_SWX AND 16L
Mode_SWX_Parity		= Mode_SWX_Parity/16L

Spectrum_HF = replicate(-1L,318)			;full power spectrum (approximate dBs (relative))


;;j = Mode_SWX_parity
;;for i = 0L, 158L do begin
;;	if Mode_SWX_Cal EQ 0 then begin
;;		if Power_HF[i] NE 255L then begin
;;			Spectrum_HF[j]	= Power_HF[i]
;;		endif else begin
;;			Spectrum_HF[j]	= 0L
;;		endelse
;;	endif else begin
;;		Spectrum_HF[j] = 0L
;;	endelse
;;	j = j+2
;;endfor

j = Mode_SWX_parity + indgen(159)*2
if Mode_SWX_Cal EQ 0 then begin
    w255 = where(Power_HF NE 255L, n255, complement=w0, ncomplement=n0)
    if n255 gt 0 then Spectrum_HF[j[w255]]	= Power_HF[w255]
    if n0   gt 0 then Spectrum_HF[j[w0]]	= 0L
endif else Spectrum_HF[j] = 0L

if n_params() eq 3 then $
  Frequency_HF = 125000L + lindgen(318) * 50000L        ;frequencies corresponding to the full power spectrum (Hz)

return

end
