;+
; Project     :	STEREO - SSC
;
; Name        :	SSC_UPDATE_PLASTIC_CDF
;
; Purpose     :	Replace PLASTIC beacon filenames with science files
;
; Category    :	STEREO, Quicklook
;
; Explanation :	Replace PLASTIC beacon filename with the names of science files
;               for the same date, if found, to produce better quality plots.
;               The routine first looks for archived science files, and then
;               for unvalidated files with the same format.
;
; Syntax      :	SSC_UPDATE_PLASTIC_CDF, FILES
;
; Examples    :	See SSC_BEACON_PLOTS
;
; Inputs      :	FILES   = The names of the beacon files to be tested against
;                         the science files.
;
; Opt. Inputs :	None
;
; Outputs     :	The array FILES is returned with the modified filenames.
;
; Opt. Outputs:	None
;
; Keywords    :	None
;
; Calls       :	BREAK_FILE, PARSE_STEREO_NAME, CONCAT_DIR
;
; Common      :	None
;
; Env. Vars.  : PLASTIC_1DMAX       = Location of PLASTIC proton data derived
;                                     from 1D Maxwellians
;
;               PLASTIC_UNVALIDATED = Location of unvalidated 1DMax data
;
; Restrictions:	None.
;
; Side effects:	None
;
; Prev. Hist. :	None
;
; History     :	Version 1, 13-Oct-2023, W. Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
pro ssc_update_plastic_cdf, files
;
;  Step through the files.  From the filename, extract the spacecraft, date,
;  and year.
;
for i=0,n_elements(files)-1 do if files[i] ne '' then begin
    break_file, files[i], disk, dir, name
    parts = strsplit(name, '_', /extract)
    sc = parse_stereo_name(parts[0], ['ahead', 'behind'])
    date = parts[3]
    year = strmid(date, 0, 4)
;
;  First, look for a corresponding file in the validated tree.  If found, then
;  replace it for the original.
;
    path = concat_dir(getenv('PLASTIC_1DMAX'), sc)
    path = concat_dir(path, '1min')
    path = concat_dir(path, year)
    test = concat_dir(path, parts[0] + '_L2_PLA_1DMax_1min_' + date + '_V*.cdf')
    test = file_search(test, count=count)
    if count gt 0 then files[i] = max(test) else begin
;
;  Otherwise, look for a corresponding file in the unvalidated tree.  If found,
;  then replace it for the original.
;
        path = concat_dir(getenv('PLASTIC_UNVALIDATED'), sc)
        path = concat_dir(path, year)
        test = concat_dir(path, parts[0] + '_L2_PLA_1DMax_1min_' + date + $
                          '_V*_unvalidated.cdf')
        test = file_search(test, count=count)
        if count gt 0 then files[i] = max(test)
    endelse
endif
;
end
