;+
; Project     :	STEREO - SSC
;
; Name        :	SSC_BROWSE_SECCHI_MPEG
;
; Purpose     :	Make daily SECCHI MPEG movies
;
; Category    :	SECCHI, Quicklook
;
; Explanation :	Uses FFMPEG to merge JPEG images into MPEG movies.  All
;               telescopes are processed for the specified date.  Existing MPEG
;               movies are regenerated only if they are older than the
;               corresponding JPEG directory.  Called from SSC_MAKE_BROWSE.
;
; Syntax      :	SSC_BROWSE_SECCHI_MPEG, DATE
;
; Examples    :	See SSC_MAKE_BROWSE
;
; Inputs      :	DATE    = The date to process.
;
; Opt. Inputs :	None.
;
; Outputs     :	MPEG files are written to the appropriate directory.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Env. Vars.  :	STEREO_BROWSE   = Top browse directory
;
; Calls       :	ANYTIM2UTC, SSW_BIN, CONCAT_DIR, ANYTIM2CAL, BREAK_FILE,
;               FILE_EXIST, MK_TEMP_FILE, GET_TEMP_DIR, MK_TEMP_DIR, DELVARX
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	Partially based on WR_MOVIE.
;
; History     :	Version 1, 05-Sep-2007, William Thompson, GSFC
;               Version 2, 07-Sep-2007, WTT, Use MK_TEMP_FILE, /RANDOM
;               Version 3, 12-Dec-2011, WTT, include _rdiff directories
;               Version 4, 28-Aug-2014, WTT, enforce # of images limit
;               Version 5, 30-Nov-2017, WTT, include COR1 pB
;               Version 6, 16-Apr-2018, WTT, include HI1 star removed
;                                            allow possibility of PNG images
;               Version 7, 24-Sep-2021, WTT, use FFMPEG
;               Version 8, 05-Aug-2022, WTT, use SSW version of FFMPEG
;
; Contact     :	WTHOMPSON
;-
;
pro ssc_browse_secchi_mpeg, date
;
stereo_browse = getenv('STEREO_BROWSE')
if n_params() eq 0 then get_utc, utc, /ext else utc = anytim2utc(date, /ext)
;
sc = ['behind','ahead']
tel = ['euvi','cor1','cor1_rdiff','cor1_pb','cor2','cor2_rdiff', $
       'hi1','hi1_rdiff','hi1_srem','hi2']
wave = ['195','171','284','304']
res = ['512','256']
nres = n_elements(res)
;
sdate = string(utc.year,  format='(I4.4)') + '/' + $
        string(utc.month, format='(I2.2)') + '/' + $
        string(utc.day,   format='(I2.2)')
filepath = concat_dir(stereo_browse, sdate)
caldate = anytim2cal(utc, form=8, /date)
;
;  Step through the various JPEG directories.
;
for isc = 0,1 do begin
    for itel=0,n_elements(tel)-1 do begin
        telescope = tel[itel]
        if telescope eq 'euvi' then nwave=n_elements(wave) else nwave=1
        for iwave=0,nwave-1 do begin
            for ires=0,nres-1 do begin
;
;  Find the directory containing the JPEG files, and make sure that it exists.
;
                jpeg_path = concat_dir(filepath, sc[isc] + '/' + telescope)
                if telescope eq 'euvi' then jpeg_path = $
                  concat_dir(jpeg_path,wave[iwave])
                jpeg_path = concat_dir(jpeg_path, res[ires])
;
;  Get the names of the jpeg files, if any.
;
                jpeg_files = file_search(concat_dir(jpeg_path,'*.jpg'), $
                                         count=count)
;
;  If no JPEG image were found, try looking for PNG images, to support the HI1
;  SREM images copied over from NRL.  Use the PNG images to create temporary
;  JPEG images.
;
                if count eq 0 then begin
                    png_files = file_search(concat_dir(jpeg_path,'*.png'), $
                                            count=count0)
                    if count0 gt 5 then begin
                        mk_temp_dir, get_temp_dir(), jpeg_path
                        tempdir = jpeg_path
                        for ifile = 0,count0-1 do begin
                            read_png, png_files[ifile], image, red, green, blue
                            sz = size(image)
                            im = bytarr(3,sz[1],sz[2])
                            im[0,*,*] = red[image]
                            im[1,*,*] = green[image]
                            im[2,*,*] = blue[image]
                            break_file, png_files[ifile], disk, dir, name
                            outfile = concat_dir(jpeg_path, name+'.jpg')
                            write_jpeg, outfile, im, true=1, quality=100
                        endfor
                        jpeg_files = file_search(concat_dir(jpeg_path,'*.jpg'), $
                                                 count=count)
                    endif
                endif
;
;  Only process the directory if at least 6 images are found.
;
                if count gt 5 then begin
                    break_file, jpeg_files, disk, dir, name, ext
                    files = name + ext
;
;  Form the the name of the output MPEG file.
;
                    mpeg_file = sc[isc] + '_' + caldate + '_' + telescope
                    if telescope eq 'euvi' then mpeg_file = $
                      mpeg_file + '_' + wave[iwave]
                    mpeg_file = mpeg_file + '_' + res[ires] + '.mpg'
                    mpeg_file = concat_dir(filepath, mpeg_file)
;
;  If the file already exists, then check the modification dates of the MPEG
;  file and corresponding JPEG files.  Only proceed if the MPEG file is older
;  than the JPEG files, or the MPEG file doesn't exist.
;
                    if file_exist(mpeg_file) then begin
                        mpeg_date = (file_info(mpeg_file)).mtime
                        jpeg_date = max((file_info(jpeg_files)).mtime)
                        test = jpeg_date gt mpeg_date
                    end else test = 1
                    if test then begin
;
;  Create a temporary directory, and copy the frames into this directory, with
;  the new names frame000.jpg, frame001.jpg, etc.
;
                        mk_temp_dir, get_temp_dir(), framedir
                        for iframe=0,n_elements(jpeg_files)-1 do begin
                            format = '(I3.3)'
                            frame_file = 'frame' + string(iframe,format=format) + '.jpg'
                            frame_file = concat_dir(framedir, frame_file)
                            file_copy, jpeg_files[iframe], frame_file
                        endfor
;
;  Execute the ffmpeg command.  Capture RESULT and ERR_RESULT to avoid
;  interfering with using 'tail' on the log file.
;
                        frame_file = concat_dir(framedir, 'frame%03d.jpg')
                        command = '$SSW/vobs/ontology/binaries/' + !version.OS + '_' + $
                                  !version.arch + '/ffmpeg -f image2 -i ' + frame_file + $
                                  ' -vcodec mpeg2video -q:v 1 -y ' + mpeg_file
                        print, command
                        spawn, command, result, err_result
                        print, result
                        print, err_result
;
;  Delete any temporary files and directories created.
;
                        file_delete, framedir, /recursive
                        if file_exist(tempdir) then begin
                            file_delete, tempdir, /recursive
                            delvarx, tempdir
                        endif
;
                    endif       ;Path newer than MPEG file
                endif           ;Path exists
            endfor              ;ires
        endfor                  ;iwave
    endfor                      ;itel
endfor                          ;isc
;
return
end
