;+
; Project     :	STEREO - SSC
;
; Name        :	SSC_MAKE_CARR
;
; Purpose     :	Create browse pages by Carrington rotation
;
; Category    :	STEREO, Quicklook
;
; Explanation :	Generate the STEREO browse pages by Carrington rotation
;
; Syntax      :	SSC_MAKE_CARR  [, CARR_ROT ]  [, /NODATA ]
;
; Examples    :	SSC_MAKE_CARR           ;Makes browse page for latest rotation
;
; Inputs      :	None required.
;
; Opt. Inputs :	CARR_ROT = The Carrington rotation to process, in half-rotation
;                          increments.
;
; Outputs     :	Generates plots and web pages.
;
; Opt. Outputs:	None.
;
; Keywords    :	NODATA  = Used recursively to create default pages for previous
;                         and next semi-rotations, if they don't exist yet.
;
;               NOPROCESSING = Don't create plots.
;
; Env. Vars.  : STEREO_BROWSE             = Top browse directory
;               STEREO_BEACON_DATA        = Location of beacon data
;               UPDATE_BLDG21_BROWSE_CARR = Script to update building 21 copy
;
; Calls       :	CONCAT_DIR, GET_UTC, GET_STEREO_CARR_ROT, MK_DIR, SET_PLOT,
;               SSC_INSITU_BEACON_CARR, TVREAD, WRITE_GIF
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 01-Jul-2010, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
pro ssc_make_carr, carr_rot, nodata=nodata, noprocessing=noprocessing
;
stereo_browse = getenv('STEREO_BROWSE')
stereo_browse_carr = concat_dir(stereo_browse, 'carrington')
;
;  If CARR_ROT is not defined, then process the most recent complete rotation,
;  in half rotation increments.
;
if n_elements(carr_rot) eq 0 then begin
    get_utc, today
    carr_rot = floor(2*get_stereo_carr_rot(today, 'Earth')) / 2. - 1
endif
;
srot = string(carr_rot,     format='(F6.1)')  &  strput, srot, 'p', 4
prev = string(carr_rot-0.5, format='(F6.1)')  &  strput, prev, 'p', 4
next = string(carr_rot+0.5, format='(F6.1)')  &  strput, next, 'p', 4
filepath = concat_dir(stereo_browse_carr, srot)
prevpath = concat_dir(stereo_browse_carr, prev)
nextpath = concat_dir(stereo_browse_carr, next)
if not keyword_set(nodata) then begin
    if not file_exist(filepath) then mk_dir, filepath
    if not file_exist(prevpath) then mk_dir, prevpath
    if not file_exist(nextpath) then mk_dir, nextpath
endif
;
;  Create default pages for the previous and next semi-rotations, if necessary.
;
if not keyword_set(nodata) then begin
    test = file_exist(concat_dir(filepath, 'index.shtml'))
    if not test then ssc_make_carr, carr_rot, /nodata
    if not file_exist(concat_dir(prevpath, 'index.shtml')) then $
      ssc_make_carr, carr_rot - 0.5, /nodata
    if not file_exist(concat_dir(nextpath, 'index.shtml')) then $
      ssc_make_carr, carr_rot + 0.5, /nodata
    if not test then ssc_make_carr, carr_rot, /nodata
endif
;
if keyword_set(nodata) or keyword_set(noprocessing) then goto, make_page
;
dname = !d.name
set_plot, 'Z'
device, set_resolution=[550,800]
!p.color = 0
!p.background = !d.table_size - 1
linecolor, 0, 'black'
tick_unit = 6*86400.d0         ;Six days
erase
;
ssc_insitu_beacon_carr, carr_rot, timerange=timerange, tick_unit=tick_unit, $
  smooth=3600
image = tvread(red, green, blue, /quiet)
w = where(image ne image[0], count)
if count gt 0 then begin
    filename = concat_dir(filepath, 'beacon_insitu.gif')
    write_gif, filename, image, red, green, blue
;
    device, set_resolution=[800,900]
    ssc_insitu_beacon_sep, timerange=timerange, tick_unit=tick_unit, $
      smooth=3600
    image = tvread(red, green, blue, /quiet)
    filename = concat_dir(filepath, 'beacon_sep.gif')
    write_gif, filename, image, red, green, blue
endif
;
set_plot, dname
;
;  Create the web page.
;
make_page:
openw, output, concat_dir(filepath, 'index.shtml'), /get_lun
;
;  Start by inserting the header
;
openr, header, concat_dir(stereo_browse_carr,'header.txt'), /get_lun
line = 'String'
while not eof(header) do begin
    readf, header, line
    printf, output, line
endwhile
free_lun, header
;
;  Print out a disclaimer about space weather data.
;
printf, output, ''
printf, output, '<P>These plots include data derived from the STEREO'
printf, output, 'space weather beacon telemetry, which is a very'
printf, output, 'low rate, highly compressed data stream broadcast'
printf, output, 'by the spacecraft 24 hours per day.  These data'
printf, output, 'are used for space weather forecasting.  Because'
printf, output, 'the data are produced in real-time, the calibrations'
printf, output, 'may not match those used for the final data'
printf, output, 'product.</P>'
printf, output, ''
;
;  Print out an explanation about the meaning of the Carrington rotation
;  numbers.
;
printf, output, '<P>The time ranges are calculated for the Carrington'
printf, output, 'rotation numbers as they pertain to Earth.  The data'
printf, output, 'have been smoothed by 1 hour for easier viewing.</P>'
printf, output, ''
;
;  Form the links to the previous and next semi-rotations.
;
filler = ''
for i=1,25 do filler = filler + "&nbsp;"
spacer = ''
for i=1,10 do spacer = spacer + "&nbsp;"
;
if file_exist(concat_dir(prevpath,'index.shtml')) then begin
    prevurl = '<A HREF="../' + prev + '/index.shtml">[' + $
      string(carr_rot-0.5, format='(F6.1)') + ']</A>'
end else begin
    prevurl = ''
    for i=1,8 do prevurl = prevurl + "&nbsp;"
endelse
;
if file_exist(concat_dir(nextpath,'index.shtml')) then begin
    nexturl = '<A HREF="../' + next + '/index.shtml">[' + $
      string(carr_rot+0.5, format='(F6.1)') + ']</A>'
end else begin
    nexturl = ''
    for i=1,8 do nexturl = nexturl + "&nbsp;"
endelse
;
printf, output, '<TABLE CLASS="coords" ALIGN=CENTER><TR>'
printf, output, '<TD>' + filler + '</TD><TD>' + filler + '</TD>'
printf, output, '<TD>' + filler + '</TD><TR></TR>'
printf, output, '<TR><TD></TD>'
printf, output, '<TH STYLE="font-size:x-large; text-align:center">'
printf, output, 'Carrington rotation number</TH>'
printf, output, '<TD></TD></TR>'
printf, output, '<TD STYLE="text-align:left">' + prevurl + '</TD>'
printf, output, '<TH STYLE="font-size:x-large; text-align:center">' + $
  string(carr_rot,format='(F6.1)') + '</TH>'
printf, output, '<TD STYLE="text-align:right">' + nexturl + '</TD></TR>'
printf, output, '</TABLE><P>'
printf, output
;
;  If the NODATA keyword was set, then write out a message saying there's no
;  data.
;
if keyword_set(nodata) then begin
    printf, output, '<H2 ALIGN=CENTER>NO DATA</H2><P>'
;
;  Otherwise, write out the body of the page.
;
end else begin
    printf, output, '<P ALIGN=CENTER>'
    image_file_name = 'beacon_insitu.gif'
    image_file = concat_dir(filepath, image_file_name)
    if file_exist(image_file) then begin
        printf, output, 'IMPACT/PLASTIC solar wind data<BR>'
        printf, output, '<IMG SRC="' + image_file_name + $
          '" ALT="in-situ">'
    end else printf, output, '<H2 ALIGN=CENTER>NOT YET AVAILABLE</H2>'
    printf, output, '</P>'
    printf, output, ''
;
    sep_file_name = 'beacon_sep.gif'
    sep_file = concat_dir(filepath, sep_file_name)
    if file_exist(sep_file) then begin
        printf, output, '<P ALIGN=CENTER>'
        printf, output, 'IMPACT solar energetic particle data</BR>'
        printf, output, '<IMG SRC="' + sep_file_name + $
          '" ALT="solar energetic particles"></P>'
        printf, output, ''
    endif
;
;  Repeat the navigation parameters at the bottom.
;
    printf, output, '<TABLE CLASS="coords" ALIGN=CENTER><TR>'
    printf, output, '<TD>' + filler + '</TD><TD>' + filler + '</TD>'
    printf, output, '<TD>' + filler + '</TD><TR></TR>'
    printf, output, '<TD STYLE="text-align:left">' + prevurl + '</TD>'
    printf, output, '<TH STYLE="font-size:x-large; text-align:center">' + $
      string(carr_rot,format='(F6.1)') + '</TH>'
    printf, output, '<TD STYLE="text-align:right">' + nexturl + '</TD></TR>'
    printf, output, '</TABLE><P>'
    printf, output
endelse                         ;Not NODATA
;
;  Write out the footer.
;
openr, footer, concat_dir(stereo_browse,'footer.txt'), /get_lun
line = 'String'
while not eof(footer) do begin
    readf, footer, line
    printf, output, line
endwhile
free_lun, footer, output
;
;  If the most recent Carrington rotation was processed, then create a redirect
;  page.
;
if n_params() eq 0 then begin
    openw, output, concat_dir(stereo_browse_carr, 'index.shtml'), /get_lun
    printf, output, '<META HTTP-EQUIV=REFRESH CONTENT="0; URL=' + srot + '">'
;
    openr, header, concat_dir(stereo_browse_carr,'header.txt'), /get_lun
    line = 'String'
    while not eof(header) do begin
        readf, header, line
        printf, output, line
    endwhile
    free_lun, header
;
    printf, output, "<P>If you're not automatically redirected,"
    printf, output, 'please follow this <A HREF="' + srot + '">link</A>'
    printf, output, 'to enter the carrington rotation pages.</P>'
;
    openr, footer, concat_dir(stereo_browse,'footer.txt'), /get_lun
    line = 'String'
    while not eof(footer) do begin
        readf, footer, line
        printf, output, line
    endwhile
    free_lun, footer, output
endif
;
;  If UPDATE_BLDG21_BROWSE_CARR is defined, then call the script to copy over
;  the Carrington tree.
;
update_bldg21_browse_carr = getenv('UPDATE_BLDG21_BROWSE_CARR')
if update_bldg21_browse_carr ne '' then begin
    print, 'Spawning: ' + update_bldg21_browse_carr
    spawn, update_bldg21_browse_carr
endif
;
end
