;+
; Project     :	STEREO - SSC
;
; Name        :	SSC_MAKE_EUVI_BROWSE_SYNOPTIC
;
; Purpose     :	Wrapper around SSC_EUVI_SYNOPTIC for browse data
;
; Category    :	STEREO, SECCHI, Coordinates
;
; Explanation :	This routine calls SSC_EUVI_SYNOPTIC to create the files
;               euvi_*_synoptic.gif in the browse directory.  Called from
;               SSC_MAKE_BROWSE.
;
; Syntax      :	SSC_MAKE_EUVI_BROWSE_SYNOPTIC, DATE, PATH
;
; Examples    :	See SSC_MAKE_BROWSE
;
; Inputs      :	DATE    = The date to process.
;               PATH    = Directory for the output files.
;
; Opt. Inputs :	None.
;
; Outputs     :	Creates <PATH>/euvi_195_synoptic.gif and other 3 wavelengths
;
; Opt. Outputs:	None.
;
; Keywords    : BEACON  = Process beacon files.  The output files will have
;                         "_beacon" appended to the filename.
;
;               REPLACE = Replace existing GIF files
;
;               Also allows VSO keywords to be passed, such as SITE='SAO'
;
; Calls       :	SETPLOT, SSC_EUVI_SYNOPTIC, TVREAD, CONCAT_DIR, WRITE_GIF
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 28-Jun-2012, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
pro ssc_make_euvi_browse_synoptic, date, path, beacon=beacon, $
                                   replace=replace, _extra=_extra
;
;  Set up the Z buffer.
;
dname = !d.name
setplot, 'z'
device, set_resolution=[640, 400]
!p.color = !d.table_size - 1
!p.background = 0
;
;  Step through the wavelengths.
;
waves = [195,304,171,284]
for iwave=0,3 do begin
;
;  Define the filename.
;
    swave = ntrim(waves[iwave])
    filename = concat_dir(path, 'euvi_' + swave + '_heliographic')
    if keyword_set(beacon) then filename = filename + '_beacon'
    filename = filename + '.gif'
;
;  If the file doesn't already exist, then create it.
;
    if keyword_set(replace) or ~file_exist(filename) or keyword_set(beacon) $
      then begin
        errmsg = ''
        title='EUVI/AIA ' + swave + ' Stonyhurst Heliographic (Earth-view)'
        ssc_euvi_synoptic, /color, errmsg=errmsg, title=title, dates=date, $
          wave=waves[iwave], beacon=beacon, _extra=_extra
        if errmsg eq '' then begin
            image = tvread(red, green, blue)
            write_gif, filename, image, red, green, blue
        endif
    endif
endfor
;
setplot, dname
;
end
