;+
; Project     :	STEREO - SSC
;
; Name        :	SSC_INGEST_ANCILLARY
;
; Purpose     :	Ingest ancillary data products by year
;
; Category    :	STEREO, Archive
;
; Explanation :	This routine moves files from the rsync copy of the MOC/SDS
;               directories to the website, organized by year.
;
; Syntax      :	SSC_INGEST_ANCILLARY, DIRECTORY, EXTENSION
;
; Examples    :	SSC_INGEST_ANCILLARY, 'moc_status_report', 'pdf', /ahead_only
;
; Inputs      :	DIRECTORY = The directory to be ingested.  There will be
;                           normally be one each for ahead and behind.
;
;               EXTENSION = The extension for the files in the directory.
;
; Opt. Inputs :	None.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	AHEAD_ONLY = If set, then only process the ahead directories.
;                            This option is for directories where the behind
;                            copy is a logical link to the ahead copy.
;
;               MTIME      = Number of days to search for new files.  The
;                            default is 30 days
;
;               IYEAR      = The part of the filename containing the year.
;                            This is normally 1 (eg. behind_2010_130_03.ws)
;                            unless /AHEAD_ONLY is set, in which case it is
;                            normally 0 (e.g. 2011_262_01 msr.pdf).
;
; Env. Vars.  : SSC_MOC_SDS_TOP  = Top directory for rsync copy of SDS
;               SSC_MOC_SDS_HTML = Top directory for online copy
;
; Calls       :	CONCAT_DIR, DIR_EXIST, MK_DIR, BREAK_FILE, 
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 29-Sep-2011, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
pro ssc_ingest_ancillary, directory, extension, ahead_only=ahead_only, $
                          mtime=k_mtime, iyear=k_iyear
                          
on_error, 2
;
if n_elements(k_mtime) eq 1 then mtime = k_mtime else mtime = 30
;
;  Get the input and output directories.  
;
moc_sds_top = getenv('SSC_MOC_SDS_TOP')
if moc_sds_top eq '' then message, 'SSC_MOC_SDS_TOP not defined'
moc_sds_html = getenv('SSC_MOC_SDS_HTML')
if moc_sds_html eq '' then message, 'SSC_MOC_SDS_HTML not defined'
;
;  Set up the spacecraft to process, and determine the position of the year in
;  the filename.
;
if keyword_set(ahead_only) then begin
    sc = 'ahead' 
    iyear = 0
end else begin
    sc = ['ahead','behind']
    iyear = 1
endelse
if n_elements(k_iyear) eq 1 then iyear = k_iyear
;
;  Step through the spacecraft.
;
for isc=0,n_elements(sc)-1 do begin
;
;  Form the path to the input and output directories.
;
    path = sc[isc] + '/data_products/' + directory
    inpath  = concat_dir(moc_sds_top,  path)
    outtop = concat_dir(moc_sds_html, path)
    if not dir_exist(outtop) then mk_dir, outtop
;
;  Find all the files within the relevant time period.
;
    spawn, 'find ' + inpath + ' -mtime -' + ntrim(mtime) + ' -name \' + $
      form_filename('*',extension), files
    if files[0] eq '' then nfiles = 0 else nfiles = n_elements(files)
;
;  For each file, extract the year.
;
    for ifile=0,nfiles-1 do begin
        break_file, files[ifile], disk, dir, name, ext
        parts = strsplit(name,'_',/extract)
        year = parts[iyear]
;
;  Make sure the year directory exists.
;
        outpath = concat_dir(outtop,year)
        if not dir_exist(outpath) then mk_dir, outpath
        outfile = concat_dir(outpath, name+ext)
;
;  If the file doesn't already exist, then copy it over.
;
        if not file_exist(outfile) then begin
            command = 'cp -fp "' + files[ifile] + '" "' + outfile + '"'
            print, command
            spawn, command
        endif
    endfor                      ;ifile
endfor                          ;isc
;
end
