;+
; Project     :	STEREO - SSC
;
; Name        :	stereo_ic_get_ace
;
; Purpose     :	Get ACE orbital GSE coordinates during STEREO conjunction
;
; Category    :	Orbit
;
; Explanation : Returns orbital coordinates of ACE spacecraft in the geocentric
;               solar ecliptic plane for plotting against STEREO's position
;               during inferior solar conjunction.  Reads coordinates from CDF
;               file.  If necessary, the CDF orbit file is downloaded from the
;               web.  Over long periods, reads from SPICE .bsp file.
;
; Syntax      :	STEREO_IC_GET_ACE, DATE, COORD
;
; Examples    :	stereo_ic_get_ace, '2023-08-14T12', ace_coord
;
; Inputs      :	DATE    = Date/time to retrieve coordinates for.
;
; Opt. Inputs :	None.
;
; Outputs     :	COORD   = GSE coordinates in kilometers.
;
; Opt. Outputs:	None.
;
; Env. Vars   : STEREO_IC_DATA = Directory containing the ephemeris data for
;                                the inferior solar conjunction.  The ACE data
;                                will be in the "ace" subdirectory.  If not
;                                defined, the present directory is used
;                                instead.
;
; Keywords    :	AU      = If set, then return data in AU (default KM).
;
;               ERRMSG = If defined and passed, then any error messages will be
;                        returned to the user in this parameter rather than
;                        depending on the MESSAGE routine in IDL.  If no errors
;                        are encountered, then a null string is returned.  In
;                        order to use this feature, ERRMSG must be defined
;                        first, e.g.
;
;                               ERRMSG = ''
;                               STEREO_IC_GET_ACE, ERRMSG=ERRMSG, ...
;                               IF ERRMSG NE '' THEN ...
;
;               Any keywords used by INTERPOL can also be passed.
;
; Calls       :	ANYTIM2UTC, UTC2TAI, CONCAT_DIR, CDF2TAI, WCS_AU,
;               LOAD_SUNSPICE_GEN, GET_SUNSPICE_COORD
;
; Common      :	None
;
; Restrictions:	Must have write access to directory containing CDF files.
;
; Side effects:	Returns array of NaN values when data not found.
;
; Prev. Hist. :	None
;
; History     :	Version 1, 2-Jun-2022, William Thompson, GSFC
;               Version 2, 8-Aug-2022, WTT, remirror if necessary.
;               Version 3, 22-Aug-2022, WTT, read SPICE .bsp file
;
; Contact     :	WTHOMPSON
;-
;
pro stereo_ic_get_ace, date, coord, au=au, errmsg=errmsg, _extra=_extra
;
;  Interpret the date, and extract the year (ext.year), and the TAI time for
;  interpolation.
;
ext = anytim2utc(date, /ext)
tai0 = utc2tai(ext)
;
;  Look for the CDF file associated with the requested year.
;
ace_dir = concat_dir(getenv('STEREO_IC_DATA'), 'ace')
test_string = 'ac_or_ssc_' + ntrim(ext.year)
file = file_search(concat_dir(ace_dir, test_string + '*.cdf'), count=count)
;
;  If the CDF file was not found, then attempt to copy it over from the web.
;  First, get a list of files, and search for the test string.
;
ntries = 0
mirror_file:
if count eq 0 then begin
    ace_path = 'https://cdaweb.gsfc.nasa.gov/pub/data/ace/orbit/' + $
               'level_2_cdaweb/or_ssc/'
    spawn, 'wget --no-check-certificate ' + ace_path + ' -O -', temp
    name = ''
    for i=0,n_elements(temp)-1 do begin
        j1 = strpos(temp[i], test_string)
        if j1 gt 0 then begin
            j0 = strpos(temp[i], 'href="') + 6
            name = strmid(temp[i], j0, 26)
        endif
    endfor
;
;  If the test string was found, then copy over the file.  If a .1 version
;  exists, then rename it.
;
    if name ne '' then begin
        Print, 'Mirroring ' + name
        spawn, 'wget --no-check-certificate ' + ace_path + name + ' -P ' + $
               ace_dir
        file = concat_dir(ace_dir, name)
        if file_exist(file+'.1') then file_move, file+'.1', file, /overwrite
        if file_exist(file) then count = 1
    endif
endif
;
;  If no file was found, then try reading the SPICE .bsp file.
;
if count eq 0 then goto, read_bsp_file
;
;  Otherwise, select the file with the highest version number.
;
if count gt 1 then file = max(file)
;
;  Read the coordinate information from the file.
;
file = file[0]
id = cdf_open(file)
cdf_control, id, get_var_info=info, variable='Epoch'
nrec = info.maxrec + 1
cdf_varget, id, 'Epoch', epoch, rec_count=nrec
cdf_varget, id, 'XYZ_GSE', xyz_gse, rec_count=nrec
cdf_close, id
;
;  If the requested date is more than a day past the last value in the file,
;  then try remirroring it.
;
tai = cdf2tai(epoch)
if max(tai) lt (tai0 - 86400.d0) then begin
    if ntries gt 0 then begin
        message = 'Beyond limit of CDF file'
        goto, read_bsp_file
    end else begin
        count = 0
        ntries = 1
        goto, mirror_file
    endelse
endif
;
;  Interpolate to the requested time.
;
coord = xyz_gse[*,0]
for i=0,2 do coord[i] = interpol(xyz_gse[i,*], tai, tai0, _extra=_extra)
if keyword_set(au) then coord = coord / wcs_au(unit='km')
;
return
;
;  Try reading SPICE .bsp file.
;
read_bsp_file:
utc = anytim2utc(date, /ccsds)
if utc ge '2024-02-11T07' then begin
    message = 'Provided date beyond ephemeris range'
    goto, handle_error
endif
;
file = concat_dir(ace_dir, 'ACE_gmat_spice_ephem_600s_20220721_213540.bsp')
if not file_exist(file) then begin
    message = 'SPICE ephemeris not found'
    goto, handle_error
endif
load_sunspice_gen
cspice_furnsh, file
coord = get_sunspice_coord(utc, '-92', system='gse', /novelocity, au=au)
cspice_unload, file
;
return
;
;  Error handling point.
;
handle_error:
if n_elements(errmsg) eq 0 then message, message else $
  errmsg = 'stereo_ic_get_ace: ' + message
coord = replicate(!values.d_nan, 3)
;
end
