;+
; Project     :	STEREO - SSC
;
; Name        :	stereo_ic_get_soho
;
; Purpose     :	Get SOHO orbital GSE coordinates during STEREO conjunction
;
; Category    :	Orbit
;
; Explanation : Returns orbital coordinates of SOHO spacecraft in the
;               geocentric solar ecliptic plane for plotting against STEREO's
;               position during inferior solar conjunction.  Over long periods,
;               reads from SPICE .bsp file.
;
; Syntax      :	STEREO_IC_GET_SOHO, DATE, COORD
;
; Examples    :	stereo_ic_get_soho, '2023-08-14T12', soho_coord
;
; Inputs      :	DATE    = Date/time to retrieve coordinates for.
;
; Opt. Inputs :	None.
;
; Outputs     :	COORD   = GSE coordinates in kilometers.
;
; Opt. Outputs:	None.
;
; Env. Vars   : STEREO_IC_DATA = Directory containing the ephemeris data for
;                                the inferior solar conjunction.  The SOHO data
;                                will be in the "soho" subdirectory.  If not
;                                defined, the present directory is used
;                                instead.
;
; Keywords    :	AU      = If set, then return data in AU (default KM).
;
;               ERRMSG = If defined and passed, then any error messages will be
;                        returned to the user in this parameter rather than
;                        depending on the MESSAGE routine in IDL.  If no errors
;                        are encountered, then a null string is returned.  In
;                        order to use this feature, ERRMSG must be defined
;                        first, e.g.
;
;                               ERRMSG = ''
;                               STEREO_IC_GET_SOHO, ERRMSG=ERRMSG, ...
;                               IF ERRMSG NE '' THEN ...
;
; Calls       :	GET_SUNSPICE_COORD, CONCAT_DIR
;
; Common      :	None
;
; Restrictions:	None
;
; Side effects: Returns array of NaN values when data not found.
;
; Prev. Hist. :	None
;
; History     :	Version 1, 22-Aug-2022, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
pro stereo_ic_get_soho, date, coord, au=au, errmsg=errmsg
;
message = ''
coord = get_sunspice_coord(date, 'soho',  system='GSE', /novelocity, au=au, $
                           errmsg=message)
;
;  If the normal SOHO ephemerides don't work, then try the long term ephemeris.
;
if message ne '' then begin
    soho_dir = concat_dir(getenv('STEREO_IC_DATA'), 'soho')
    file = concat_dir(soho_dir, '6YREPH_2021.bsp')
    cspice_furnsh, file
    message = ''
    coord = get_sunspice_coord(date, 'soho',  system='GSE', /novelocity, $
                               au=au, errmsg=message)
    cspice_unload, file
    if message ne '' then goto, handle_error
endif
;
return
;
;  Error handling point.
;
handle_error:
if n_elements(errmsg) eq 0 then message, message else $
  errmsg = 'stereo_ic_get_soho: ' + message
coord = replicate(!values.d_nan, 3)
;
end
