;+
; Project     :	STEREO - SSC
;
; Name        :	stereo_ic_get_wind
;
; Purpose     :	Get WIND orbital GSE coordinates during STEREO conjunction
;
; Category    :	Orbit
;
; Explanation : Returns orbital coordinates of WIND spacecraft in the
;               geocentric solar ecliptic plane for plotting against STEREO's
;               position during inferior solar conjunction.  Reads coordinates
;               from CDF file.  If necessary, the CDF orbit file is downloaded
;               from the web.  Over long periods, reads from SPICE .bsp file.
;
; Syntax      :	STEREO_IC_GET_WIND, DATE, COORD
;
; Examples    :	stereo_ic_get_wind, '2023-08-14T12', wind_coord
;
; Inputs      :	DATE    = Date/time to retrieve coordinates for.
;
; Opt. Inputs :	None.
;
; Outputs     :	COORD   = GSE coordinates in kilometers.
;
; Opt. Outputs:	None.
;
; Env. Vars   : STEREO_IC_DATA = Directory containing the ephemeris data for
;                                the inferior solar conjunction.  The WIND data
;                                will be in the "wind" subdirectory.  If not
;                                defined, the present directory is used
;                                instead.
;
; Keywords    :	AU      = If set, then return data in AU (default KM).
;
;               ERRMSG = If defined and passed, then any error messages will be
;                        returned to the user in this parameter rather than
;                        depending on the MESSAGE routine in IDL.  If no errors
;                        are encountered, then a null string is returned.  In
;                        order to use this feature, ERRMSG must be defined
;                        first, e.g.
;
;                               ERRMSG = ''
;                               STEREO_IC_GET_WIND, ERRMSG=ERRMSG, ...
;                               IF ERRMSG NE '' THEN ...
;
;               Any keywords used by INTERPOL can also be passed.
;
; Calls       :	ANYTIM2UTC, UTC2TAI, ANYTIM2CAL, CONCAT_DIR, CDF2TAI, WCS_AU,
;               LOAD_SUNSPICE_GEN, GET_SUNSPICE_COORD
;
; Common      :	None
;
; Restrictions:	Must have write access to directory containing CDF files.
;
; Side effects: Returns array of NaN values when data not found.
;
; Prev. Hist. :	None
;
; History     :	Version 1, 2-Jun-2022, William Thompson, GSFC
;               Version 2, 22-Aug-2022, WTT, read SPICE .bsp file.
;
; Contact     :	WTHOMPSON
;-
;
pro stereo_ic_get_wind, date, coord, au=au, errmsg=errmsg, _extra=_extra
;
;  Interpret the date, and extract the year (ext.year), and the TAI time for
;  interpolation.
;
ext = anytim2utc(date, /ext)
tai0 = utc2tai(ext)
sdate = anytim2cal(ext, form=8, /date)
;
;  Look for the CDF file associated with the requested date.
;
wind_dir = concat_dir(getenv('STEREO_IC_DATA'), 'wind')
test_string = 'wi_or_pre_' + sdate
file = file_search(concat_dir(wind_dir, test_string + '*.cdf'), count=count)
;
;  If the CDF file was not found, then attempt to copy it over from the web.
;  First, get a list of files, and search for the test string.
;
if count eq 0 then begin
    wind_path = 'https://cdaweb.gsfc.nasa.gov/pub/data/wind/orbit/pre_or/' + $
      ntrim(ext.year) + '/'
    spawn, 'wget --no-check-certificate ' + wind_path + ' -O -', temp
    name = ''
    for i=0,n_elements(temp)-1 do begin
        j1 = strpos(temp[i], test_string)
        if j1 gt 0 then begin
            j0 = strpos(temp[i], 'href="') + 6
            name = strmid(temp[i], j0, 26)
        endif
    endfor
;
;  If the test string was found, then copy over the file.
;
    if name ne '' then begin
        spawn, 'wget --no-check-certificate ' + wind_path + name + ' -P ' + $
               wind_dir
        file = concat_dir(wind_dir, name)
        if file_exist(file) then count = 1
    endif
endif
;
;  If no file was found, then try reading the SPICE .bsp file.
;
if count eq 0 then goto, read_bsp_file
;
;  Otherwise, select the file with the highest version number.
;
if count gt 1 then file = max(file)
;
;  Read the coordinate information from the file.
;
id = cdf_open(file)
cdf_control, id, get_var_info=info, variable='Epoch'
nrec = info.maxrec + 1
cdf_varget, id, 'Epoch', epoch, rec_count=nrec
cdf_varget, id, 'GSE_POS', gse_pos, rec_count=nrec
cdf_close, id
;
;  Interpolate to the requested time.
;
tai = cdf2tai(reform(epoch[0,*]))
coord = gse_pos[*,0]
for i=0,2 do coord[i] = interpol(gse_pos[i,*], tai, tai0, _extra=_extra)
if keyword_set(au) then coord = coord / wcs_au(unit='km')
;
return
;
;  Try reading SPICE .bsp file.
;
read_bsp_file:
utc = anytim2utc(date, /ccsds)
if utc ge '2024-04-04T03' then begin
    message = 'Provided date beyond ephemeris range'
    goto, handle_error
endif
;
file = concat_dir(wind_dir, 'WIND_gmat_spice_ephem_600s_20220721_224411.bsp')
load_sunspice_gen
cspice_furnsh, file
coord = get_sunspice_coord(utc, 'wind', system='gse', /novelocity, au=au)
cspice_unload, file
;
return
;
;  Error handling point.
;
handle_error:
if n_elements(errmsg) eq 0 then message, message else $
  errmsg = 'stereo_ic_get_wind: ' + message
coord = replicate(!values.d_nan, 3)
;
end
