;+
; Project     :	STEREO - SSC
;
; Name        :	STEREO_IC_MAKE_GIF
;
; Purpose     :	Make GIF plot showing STEREO-A and L1 during inf. conj.
;
; Category    :	STEREO, Orbit, Graphics
;
; Explanation :	Front end to the STEREO_IC_PLOT routine to create a GIF image
;               showing the ecliptic positions of the STEREO-A spacecraft along
;               with those of several located at L1 during STEREO-A'
;               inferior conjunction in August 2023.  Used by the STEREO
;               Inferior Conjunction Tool web page.
;
; Syntax      :	STEREO_IC_MAKE_GIF, DATE, FILENAME
;
; Examples    :	GET_UTC, UTC  &  STEREO_IC_MAKE_GIF, UTC, 'test.gif'
;
; Inputs      :	DATE     = The date/time to be plotted.  Can by in any format
;                          supported by ANYTIM2UTC.
;
;               FILENAME = The name of the file to be written.  The default
;                          extension is ".gif".
;
; Opt. Inputs :	None
;
; Outputs     :	A GIF file is created showing the spacecraft positions.
;
; Opt. Outputs:	None.
;
; Keywords    :	RESOLUTION = Two element array giving the size of the GIF image
;                            to create.  Defaults to [680,840].
;
;               INFOFILE = Name of accompanying information file.
;
; Calls       :	SETPLOT, STEREO_IC_PLOT, TVREAD, WRITE_GIF, FORM_FILENAME,
;               STEREO_IC_WRITE_SUMMARY
;
; Common      :	None
;
; Restrictions:	None
;
; Side effects:	Loads the STEREO ephemerides if not already loaded.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 24-Aug-2022, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
pro stereo_ic_make_gif, date, filename, resolution=k_resolution, $
                        infofile=infofile, _extra=_extra
;
if n_elements(k_resolution) eq 2 then resolution = k_resolution else $
  resolution = [680,840]
;
dname = !d.name
setplot, 'z'
device, set_resolution=resolution
stereo_ic_plot, date, _extra=_extra
img = tvread(r,g,b)
write_gif, form_filename(filename, '.gif'), img, r, g, b
setplot, dname
;
if n_elements(infofile) eq 1 then $
  stereo_ic_write_summary, date, filename=form_filename(infofile, '.txt'), $
                           /au, _extra=_extra
;
end
