;+
; Project     :	STEREO - SSC
;
; Name        :	stereo_ic_plot
;
; Purpose     :	PLOT STEREO-A coordinates along with L1 spacecraft
;
; Category    :	Orbit
;
; Explanation : Plots the position of the STEREO-A spacecraft along with those
;               of several located at L1 during STEREO-A's inferior
;               conjunction in August 2023.
;
; Syntax      :	STEREO_IC_PLOT, DATE
;
; Examples    :	stereo_ic_plot, '2023-08-17'
;
; Inputs      :	DATE    = Date/time to retrieve coordinates for.
;
; Opt. Inputs :	None
;
; Outputs     :	None
;
; Opt. Outputs:	None
;
; Env. Vars   : None
;
; Keywords    :	TITLE   = Plot title for the first panel.  If not passed, then
;                         a title is formed from the date.
;
;               Other standard plotting keywords can also be passed.
;
;               NOLABELS = If set, then don't show the labels.
;
;               WHITE_BG  = If set, then plot against a white background.
;
;               ERRMSG = If defined and passed, then any error messages will be
;                        returned to the user in this parameter rather than
;                        depending on the MESSAGE routine in IDL.  If no errors
;                        are encountered, then a null string is returned.  In
;                        order to use this feature, ERRMSG must be defined
;                        first, e.g.
;
;                               ERRMSG = ''
;                               STEREO_IC_PLOT, ERRMSG=ERRMSG, ...
;                               IF ERRMSG NE '' THEN ...
;
; Calls       :	ANYTIM2UTC, STEREO_IC_GET_ACE, STEREO_IC_GET_WIND,
;               STEREO_IC_GET_DSCOVR, STEREO_IC_GET_SOHO, GET_SUNSPICE_COORD,
;               LINECOLOR, CIRCLE_SYM, SETVIEW, SETSCALE, LABEL_SYMBOL
;
; Common      :	None
;
; Restrictions:	DATE should be close to the date of inferior conjunction in
;               mid-August 2023.
;
; Side effects:	None
;
; Prev. Hist. :	None
;
; History     :	Version 1, 23-Aug-2022, William Thompson, GSFC
;               Version 2, 29-Aug-2022, WTT, add WHITE_BG keyword
;                       Adjust colors for background
;
; Contact     :	WTHOMPSON
;-
;
pro stereo_ic_plot, date, title=title, nolabels=nolabels, errmsg=errmsg, $
                    white_bg=white_bg, _extra=_extra
;
;  If TITLE was not passed, then use the date as the title.
;
message = ''
if (datatype(title) ne 'STR') then $
  title = anytim2utc(date, /vms, /truncate, errmsg=message)
if message ne '' then goto, handle_error
;
;  Get the location of the ACE spacecraft.
;
message = ''
stereo_ic_get_ace, date, ace, /au, errmsg=message
if message ne '' then message, message, /continue
;
;  Get the location of the WIND spacecraft.
;
message = ''
stereo_ic_get_wind, date, wind, /au, errmsg=message
if message ne '' then message, message, /continue
;
;  Get the location of the DSCOVR spacecraft.
;
message = ''
stereo_ic_get_dscovr, date, dscovr, /au, errmsg=message
if message ne '' then message, message, /continue
;
;  Get the location of the SOHO spacecraft.
;
message = ''
stereo_ic_get_soho, date, soho, /au, errmsg=message
if message ne '' then message, message, /continue
;
;  Get the coordinates of STEREO-A and the Moon.
;
ahead = get_sunspice_coord(date, 'ahead', system='GSE', /novelocity, /au)
moon  = get_sunspice_coord(date, 'moon',  system='GSE', /novelocity, /au)
;
;  Determine the minimum and maximum values of the coordinates.
;
xmin = min([-0.0025, ace[0], wind[0], dscovr[0], soho[0], ahead[0]], max=xmax)
xmin = xmin - 0.001
xmax = xmax + 0.001
ymin = min([ace[1], wind[1], dscovr[1], soho[1], moon[1], ahead[1]], max=ymax)
zmin = min([ace[2], wind[2], dscovr[2], soho[2], moon[2], ahead[2]], max=zmax)
;
;  Define a combined range based on the Y and Z values.
;
yzmax = (abs(ymin) > abs(ymax) > abs(zmin) > abs(zmax)) * 1.1
yzmin = -yzmax
;
;  Load the color table.
;
loadct,0
grey  = !d.table_size * 3 / 4
grey2 = !d.table_size / 2
blue = 1        &  linecolor,blue,'blue'
yellow = 2      &  linecolor,yellow,'yellow'
red = 3         &  linecolor,red,'red'
orange = 4      &  linecolor,orange,'orange'
purple = 5      &  linecolor,purple,'purple'
green = 6       &  linecolor,green,'green'
turquoise = 7   &  linecolor,turquoise,'turquoise'
;
;  If a white background is requested, then manipulate the !P.COLOR and
;  !P.BACKGROUND environment variables.
;
background_set = 0
if keyword_set(white_bg) and (!p.background eq 0) then begin
    background_set = 1
    background_save = !p.background
    color_save = !p.color
    !p.color = background_save
    !p.background = color_save
endif
;
;  If a black background, lighten up the blue color.
;
if !p.background le 0.2*!d.table_size then begin
    tvlct, rr, gg, bb, /get
    rr[blue] = 0.2 * !d.table_size
    gg[blue] = 0.4 * !d.table_size
    tvlct, rr, gg, bb
;
;  Otherwise, if a white background, then darken the turquoise color.
;
end else if !p.background ge 0.8*!d.table_size then begin
    tvlct, rr, gg, bb, /get
    gg[turquoise] = 0.8*gg[turquoise]
    bb[turquoise] = 0.8*bb[turquoise]
    tvlct, rr, gg, bb
endif
;
;  Define a circle symbol.
;
circle_sym, /fill
;
;  Plot the X-Y positions in the ecliptic plane.
;
erase
setview, 1, 1, 1, 3
setscale, xmin, xmax, yzmin, yzmax, /noadjust
plot, [xmin, xmax], [yzmin, yzmax], /nodata, xtitle='GSE X (AU)', $
      ytitle='GSE Y (AU)', title=title, _extra=_extra
plots, [0,0], [0,0], psym=8, color=green, symsize=2
plots, moon[0], moon[1], psym=8, color=grey
plots, ace[0], ace[1], psym=8, color=orange, symsize=0.75
plots, wind[0], wind[1], psym=8, color=blue, symsize=0.75
plots, dscovr[0], dscovr[1], psym=8, color=purple, symsize=0.75
plots, soho[0], soho[1], psym=8, color=turquoise, symsize=0.75
plots, ahead[0], ahead[1], psym=8, color=red, symsize=0.75
setscale
;
;  Add the labels.
;
if not keyword_set(nolabels) then begin
    y0 = !y.crange[0]
    yd = !y.crange[1] - y0
    if !x.crange[0] lt -0.04 then x0 = !x.crange[0]*0.75 else x0 = 0.025
    label_symbol, x0, y0 + 0.9*yd, 8, 'STEREO Ahead', color=red, symsize=0.75
    label_symbol, x0, y0 + 0.8*yd, 8, 'ACE', color=orange, symsize=0.75
    label_symbol, x0, y0 + 0.7*yd, 8, 'WIND', color=blue, symsize=0.75
    label_symbol, x0, y0 + 0.6*yd, 8, 'DISCOVR', color=purple, symsize=0.75
    label_symbol, x0, y0 + 0.5*yd, 8, 'SOHO', color=turquoise, symsize=0.75
    label_symbol, x0, y0 + 0.3*yd, 8, 'Earth', color=green, symsize=0.75
    label_symbol, x0, y0 + 0.2*yd, 8, 'Moon', color=grey, symsize=0.75
endif
;
;  Plot the X-Z positions out of the ecliptic plane.
;
setview, 1, 1, 2, 3
setscale, xmin, xmax, yzmin, yzmax, /noadjust
plot, [xmin, xmax], [yzmin, yzmax], /nodata, xtitle='GSE X (AU)', $
      ytitle='GSE Z (AU)', _extra=_extra
plots, [0,0], [0,0], psym=8, color=green, symsize=2
plots, moon[0], moon[2], psym=8, color=grey
plots, ace[0], ace[2], psym=8, color=orange, symsize=0.75
plots, wind[0], wind[2], psym=8, color=blue, symsize=0.75
plots, dscovr[0], dscovr[2], psym=8, color=purple, symsize=0.75
plots, soho[0], soho[2], psym=8, color=turquoise, symsize=0.75
plots, ahead[0], ahead[2], psym=8, color=red, symsize=0.75
ycrange = !y.crange
setscale
;
;  Plot the Y-Z positions looking toward the Sun.
;
setview, 1, 1, 3, 3
setscale, ycrange[1], ycrange[0], ycrange[0], ycrange[1], /noborder
plot, [ymin, ymax], [zmin, zmax], /nodata, xtitle='GSE Y (AU)', $
      ytitle='GSE Z (AU)', _extra=_extra
plots, [0,0], [0,0], psym=8, color=green, symsize=2
plots, moon[1], moon[2], psym=8, color=grey
plots, ace[1], ace[2], psym=8, color=orange, symsize=0.75
plots, wind[1], wind[2], psym=8, color=blue, symsize=0.75
plots, dscovr[1], dscovr[2], psym=8, color=purple, symsize=0.75
plots, soho[1], soho[2], psym=8, color=turquoise, symsize=0.75
plots, ahead[1], ahead[2], psym=8, color=red, symsize=0.75
setscale
setview
;
goto, finish
;
;  Error handling point.
;
handle_error:
if n_elements(errmsg) eq 0 then message, message else $
  errmsg = 'stereo_ic_plot: ' + message
;
;  Exit point.  If the !P variables were modified, return then to their
;  original values.
;
finish:
if background_set then begin
    !p.color = color_save
    !p.background = background_save
endif
;
return
end
