;+
; Project     :	STEREO - SSC
;
; Name        :	stereo_ic_read_stk
;
; Purpose     :	Get DSCOVR orbital GSE coordinates during STEREO conjunction
;
; Category    :	Orbit
;
; Explanation : Reads coordinates from STK file.
;
; Syntax      :	STEREO_IC_READ_STK, FILENAME, DATE, COORD
;
; Examples    :	file = 'DSCOVR_LongTerm_20222271200_20242271200_TVHF492.e'
;               stereo_ic_read_stk, file, date, dscovr_coord
;
; Inputs      :	FILENAME = Name of STK file.
;
; Opt. Inputs :	None.
;
; Outputs     :	DATE    = Array of data values.
;
;               COORD   = GSE coordinates in kilometers.
;
; Opt. Outputs:	None
;
; Env. Vars   : None
;
; Keywords    :	TAI     = If set, then DATE return as TAI instead of UTC
;
;               AU      = If set, then return data in AU (default KM).
;
;               ERRMSG = If defined and passed, then any error messages will be
;                        returned to the user in this parameter rather than
;                        depending on the MESSAGE routine in IDL.  If no errors
;                        are encountered, then a null string is returned.  In
;                        order to use this feature, ERRMSG must be defined
;                        first, e.g.
;
;                               ERRMSG = ''
;                               STEREO_IC_READ_STK, ERRMSG=ERRMSG, ...
;                               IF ERRMSG NE '' THEN ...
;
; Calls       :	UTC2TAI, TAI2UTC, WCS_AU, LOAD_SUNSPICE_GEN,
;               CONVERT_SUNPSICE_COORD
;
; Common      :	None
;
; Restrictions:	None
;
; Side effects:	None
;
; Prev. Hist. :	None
;
; History     :	Version 1, 19-Aug-2022, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
pro stereo_ic_read_stk, filename, date, coord, tai=k_tai, au=au
;
openr, unit, filename, /get_lun
;
line = 'string'
while strpos(line, 'EphemerisTimePosVel') lt 0 do begin
    readf, unit, line
    if strpos(line, 'NumberOfEphemerisPoints') ge 0 then begin
        line = strtrim(line, 2)
        words = strsplit(line, /extract)
        numpoints = long(words[1])
    end else if strpos(line, 'Time of first point') ge 0 then begin
        line = strtrim(line, 2)
        words = strsplit(line, /extract)
        tai0 = utc2tai(words[5]+'-'+words[6]+'-'+words[7]+'T'+words[8])
    endif
endwhile
;
readf, unit, line
;
ephem = dblarr(7, numpoints)
readf, unit, ephem
;
;  Extract the date in either UTC or TAI format.
;
date = reform(ephem[0,*])
date = date - date[0] + tai0
if not keyword_set(k_tai) then date = tai2utc(date, /ccsds)
;
if keyword_set(au) then conv = wcs_au() else conv = 1000
coord = ephem[1:*,*] / conv     ;Convert meters to km or AU
load_sunspice_gen
convert_sunspice_coord, date, coord, 'gei', 'gse'
;
free_lun, unit
;
end
