;+
; Project     :	STEREO - SSC
;
; Name        :	stereo_ic_write_ancillary
;
; Purpose     :	PLOT STEREO-A coordinates along with L1 spacecraft
;
; Category    :	Orbit
;
; Explanation : Writes out ancillary information for the STEREO inferior
;               conjunction period.
;
; Syntax      :	STEREO_IC_WRITE_ANCILLARY, DATE
;
; Examples    :	stereo_ic_write_ancillary, '2023-08-17'
;
; Inputs      :	DATE = Date/time to write information for.
;
;               FILENAME = Name of output file.
;
; Opt. Inputs :	None
;
; Outputs     :	None
;
; Opt. Outputs:	None
;
; Env. Vars   : None
;
; Keywords    :	ERRMSG = If defined and passed, then any error messages will be
;                        returned to the user in this parameter rather than
;                        depending on the MESSAGE routine in IDL.  If no errors
;                        are encountered, then a null string is returned.  In
;                        order to use this feature, ERRMSG must be defined
;                        first, e.g.
;
;                               ERRMSG = ''
;                               STEREO_IC_WRITE_ANCILLARY, ERRMSG=ERRMSG, ...
;                               IF ERRMSG NE '' THEN ...
;
; Calls       :	ANYTIM2UTC, DATATYPE, GET_UTC
;
; Common      :	None
;
; Restrictions:	DATE should be close to the date of inferior conjunction in
;               mid-August 2023.
;
; Side effects:	None
;
; Prev. Hist. :	None
;
; History     :	Version 1, 23-Jun-2023, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
pro stereo_ic_write_ancillary, date, filename, errmsg=errmsg
on_error, 2
;
;  Catch any errors
;
catch, error_status
if error_status ne 0 then begin
    message = !error_state.msg
    catch, /cancel
    goto, handle_error
endif
;
;  Check the validity of the date.
;
message = ''
sdate = anytim2utc(date, /ecs, /date_only, errmsg=message)
if message ne '' then goto, handle_error
;
;  Open the output file
;
if (n_elements(filename) ne 1) or (datatype(filename) ne 'STR') then begin
    message = 'FILENAME must be a scalar string'
    goto, handle_error
endif
;
;
get_utc, today, /ecs, /date_only
use_today = sdate gt today
if use_today then begin
    sdate = today
    tdate = 'today'
end else tdate = sdate
;
openw, unit, filename, /get_lun
printf, unit, '<HR>'
printf, unit, 'Available resources for STEREO fly-by of Earth:'
printf, unit, '<UL>'
printf, unit, '<LI><A HREF="https://stereo-ssc.nascom.nasa.gov/cgi-bin/' + $
        'make_where_gif" TARGET=_blank>STEREO Orbit Tool</A>'
printf, unit, '<LI><A HREF="https://stereo-ssc.nascom.nasa.gov/where/' + $
        'StereoA-Roll-Angle.gif" TARGET="_blank">STEREO-A roll angle ' + $
        'during Fly-By</A>'
;
browse_images = 'https://stereo-ssc.nascom.nasa.gov/browse/' + sdate + '/'
printf, unit, '<LI><A HREF="' + browse_images + '" TARGET="_blank">' + $
        'STEREO browse images for ' + tdate + '</A>'
browse_plots = 'https://stereo-ssc.nascom.nasa.gov/browse/' + sdate + $
               '/insitu.shtml'
printf, unit, '<LI><A HREF="' + browse_plots + '" TARGET="_blank">' + $
        'STEREO browse plots for ' + tdate + '</A>'
printf, unit, '<LI><A HREF="https://stereo-ssc.nascom.nasa.gov/beacon/' + $
        'beacon_secchi.shtml" TARGET="_blank">Latest SECCHI beacon images</A>'
printf, unit, '<LI><A HREF="https://stereo-ssc.nascom.nasa.gov/beacon/' + $
        'beacon_insitu.shtml" TARGET="_blank">STEREO In-Situ and Radio ' + $
        'Space Weather Beacon Data'
;
printf, unit, '</UL>'
printf, unit, '<UL>'
;
printf, unit, '<LI><A HREF="https://soho.nascom.nasa.gov/data/' + $
        'realtime-images.html" TARGET="_blank">Latest SOHO Images</A>'
printf, unit, '<LI><A HREF="https://soho.nascom.nasa.gov/data/Theater/" ' + $
        'TARGET="_blank">SOHO Movie Theater</A>'
;
printf, unit, '</UL>'
printf, unit, '<UL>'
;
printf, unit, '<LI><A HREF="https://sdo.gsfc.nasa.gov/data/" ' + $
        'TARGET="_blank">SDO latest images</A>'
printf, unit, '<LI><A HREF="https://sdo.gsfc.nasa.gov/data/aiahmi/" ' + $
        'TARGET="_blank">SDO AIA/HMI Browse Data</A>'
printf, unit, '<LI><A HREF="https://sdo.gsfc.nasa.gov/assets/img/dailymov/' + $
        sdate + '/" TARGET="_blank">SDO Daily Movies for ' + tdate
;
printf, unit, '</UL>'
printf, unit, '<UL>'
;
mdate = strmid(sdate,0,4) + strmid(sdate,5,2)
kyoto_dst = 'https://wdc.kugi.kyoto-u.ac.jp/dst_realtime/' + mdate + $
            '/index.html'
printf, unit, '<LI><A HREF="' + kyoto_dst + '" TARGET="_blank">' + $
        'Kyoto Dst index for ' + tdate + '</A>'
printf, unit, '<LI><A HREF="https://www.ngdc.noaa.gov/dscovr/portal/#/" ' + $
        'TARGET="_blank">DISCOVR Space Weather Data Portal</A>'
printf, unit, '<LI><A HREF="https://www.swpc.noaa.gov/products/' + $
        'goes-x-ray-flux" TARGET="_blank">GOES X-Ray Flux</A>'
printf, unit, '<LI><A HREF="https://www.swpc.noaa.gov/products/' + $
        'wsa-enlil-solar-wind-prediction" TARGET="_blank">WSA-ENLIL ' + $
        'Solar Wind Prediction</A>'
;
printf, unit, '</UL>'
free_lun, unit
return
;
;  Error handling point.
;
handle_error:
catch, /cancel
if n_elements(errmsg) eq 0 then message, message else $
  errmsg = 'stereo_ic_write_ancillary: ' + message
;
end
