;+
; Project     :	STEREO - SSC
;
; Name        :	stereo_ic_write_summary
;
; Purpose     :	PLOT STEREO-A coordinates along with L1 spacecraft
;
; Category    :	Orbit
;
; Explanation : Prints the position of the STEREO-A spacecraft along with those
;               of several located at L1 during STEREO-A's inferior conjunction
;               in August 2023.
;
; Syntax      :	STEREO_IC_WRITE_SUMMARY, DATE
;
; Examples    :	stereo_ic_write_summary, '2023-08-17'
;
; Inputs      :	DATE = Date/time to calculate positions for.
;
; Opt. Inputs :	None
;
; Outputs     :	None
;
; Opt. Outputs:	None
;
; Env. Vars   : None
;
; Keywords    :	FILENAME = Name of output file.
;
;               AU = If set, then print data in AU (default KM).
;
;               ERRMSG = If defined and passed, then any error messages will be
;                        returned to the user in this parameter rather than
;                        depending on the MESSAGE routine in IDL.  If no errors
;                        are encountered, then a null string is returned.  In
;                        order to use this feature, ERRMSG must be defined
;                        first, e.g.
;
;                               ERRMSG = ''
;                               STEREO_IC_WRITE_SUMMARY, ERRMSG=ERRMSG, ...
;                               IF ERRMSG NE '' THEN ...
;
; Calls       :	ANYTIM2UTC, STEREO_IC_GET_ACE, STEREO_IC_GET_WIND,
;               STEREO_IC_GET_DSCOVR, STEREO_IC_GET_SOHO
;
; Common      :	None
;
; Restrictions:	DATE should be close to the date of inferior conjunction in
;               mid-August 2023.
;
; Side effects:	None
;
; Prev. Hist. :	None
;
; History     :	Version 1, 24-Aug-2022, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
pro stereo_ic_write_summary, date, filename=filename, au=au, errmsg=errmsg
;
;  Set up the header.
;
message = ''
header = '            Satellite locations on ' + $
         anytim2utc(date, /vms, /truncate, errmsg=message)
if message ne '' then goto, handle_error
;
header = [header, '']
header = [header, '                  GSE-X          GSE-Y         GSE-Z']
if keyword_set(au) then $
  header = [header, '		   (AU)           (AU)          (AU)'] else $
  header = [header, '		   (KM)           (KM)          (KM)']
header = [header, '']
;
;  If FILENAME was passed, then open the output file.
;
if datatype(filename) eq 'STR' then openw, unit, filename, /get_lun
;
;  Write out the header.
;
for i=0,n_elements(header)-1 do print, header[i]
if datatype(filename) eq 'STR' then for i=0,n_elements(header)-1 do $
  printf, unit, header[i]
;
if keyword_set(au) then format = '(A10,3F15.7)' else $
  format = '(A10,3F15.3)'
;
;  Get the coordinates of STEREO-A.
;
ahead = get_sunspice_coord(date, 'ahead', system='GSE', /novelocity, au=au)
print, 'STEREO-A', ahead, format=format
if datatype(filename) eq 'STR' then $
  printf, unit, 'STEREO-A', ahead, format=format
;
;  Get the location of the ACE spacecraft.
;
message = ''
stereo_ic_get_ace, date, ace, au=au, errmsg=message
if message ne '' then message, message, /continue else begin
    print, 'ACE', ace, format=format
    if datatype(filename) eq 'STR' then $
      printf, unit, 'ACE', ace, format=format
endelse
;
;  Get the location of the WIND spacecraft.
;
message = ''
stereo_ic_get_wind, date, wind, au=au, errmsg=message
if message ne '' then message, message, /continue else begin
    print, 'WIND', wind, format=format
    if datatype(filename) eq 'STR' then $
      printf, unit, 'WIND', wind, format=format
endelse
;
;  Get the location of the DSCOVR spacecraft.
;
message = ''
stereo_ic_get_dscovr, date, dscovr, au=au, errmsg=message
if message ne '' then message, message, /continue else begin
    print, 'DSCOVR', dscovr, format=format
    if datatype(filename) eq 'STR' then $
      printf, unit, 'DSCOVR', dscovr, format=format
endelse
;
;  Get the location of the SOHO spacecraft.
;
message = ''
stereo_ic_get_soho, date, soho, au=au, errmsg=message
if message ne '' then message, message, /continue else begin
    print, 'SOHO', soho, format=format
    if datatype(filename) eq 'STR' then $
      printf, unit, 'SOHO', soho, format=format
endelse
;
if datatype(filename) eq 'STR' then free_lun, unit
return
;
;  Error handling point.
;
handle_error:
if n_elements(errmsg) eq 0 then message, message else $
  errmsg = 'stereo_ic_write_summary: ' + message
;
end
