;+
; Project     :	STEREO - SSC
;
; Name        :	SSC_PARSE_NOAA_VIEWS
;
; Purpose     :	Compares NOAA viewperiods with DSN and other stations
;
; Category    :	STEREO, Operations, Orbit
;
; Explanation :	Reads in the viewperiods for the two NOAA stations, and
;               produces a plot comparing these viewperiods with those of
;               other stations and the DSN.
;
; Syntax      :	SSC_PARSE_NOAA_VIEWS
;
; Inputs      :	None
;
; Opt. Inputs :	None
;
; Outputs     :	Plots are generated for each date analyzed.  The file
;               noaa_views.csv contains the data used to generate the plots.
;
; Opt. Outputs:	None
;
; Keywords    :	Accepts regular plotting keywords
;
; Env. Vars.  : SSC_NOTIFY     = Mail address for notifications of errors
;               SSC_PLAN       = Directory containing DSN schedule
;               SSC_STATIONS_W = Directory containing station files
;
; Calls       :	CONCAT_DIR, DELVARX, BOOST_ARRAY, UTC2TAI, SSC_READ_VIEWPERIOD,
;               TAI2UTC, UTC2STR, UTPLOT, OUTPLOT, MAIL, SETPLOT
;
; Common      :	None
;
; Restrictions:	None
;
; Side effects:	None
;
; Prev. Hist. :	None
;
; History     :	Version 1, 27-Nov-2023, William Thompson, GSFC
;               Version 2, 28-Nov-2023, WTT, add Koganei, KWSC
;                       Put full durations in CSV file.
;
; Contact     :	WTHOMPSON
;-
;
pro ssc_parse_noaa_views, _extra=_extra
;
delvarx, outlun, dname
;
;  Get the logical variables.
;
ssc_notify = getenv('SSC_NOTIFY')
ssc_plan = getenv('SSC_PLAN')
ssc_stations = getenv('SSC_STATIONS_W')
ssc_views = concat_dir(ssc_stations, 'viewperiods')
ssc_noaa  = concat_dir(ssc_stations, 'noaa_views')
;
;  Read in the DSN schedule.
;
dsn_schedule = concat_dir(ssc_plan, 'dsn_schedule.txt')
if not file_exist(dsn_schedule) then begin
    message = 'File dsn_schedule.txt not found'
    goto, handle_error
endif
;
openr, unit, dsn_schedule, /get_lun
delvarx, dsn_start, dsn_end
line = 'string'
while not eof(unit) do begin
    readf, unit, line
    if strmid(line,0,5) eq 'Ahead' then begin
        boost_array, dsn_start, strmid(line,  8, 17)
        boost_array, dsn_end,   strmid(line, 26, 17)
    endif
endwhile
free_lun, unit
;
;  Convert to TAI.
;
dsn_start = utc2tai(dsn_start[*])
dsn_end   = utc2tai(dsn_end[*])
;
;  Read in viewperiods for the NOAA stations.
;
fb_file = concat_dir(ssc_views, 'ahead_fairbanks.view')
if not file_exist(fb_file) then begin
    message = 'File ' + fb_file + ' not found'
    goto, handle_error
endif
ssc_read_viewperiod, fb_file, fb_start, fb_end
;
wp_file = concat_dir(ssc_views, 'ahead_wallops.view')
if not file_exist(wp_file) then begin
    message = 'File ' + wp_file + ' not found'
    goto, handle_error
endif
ssc_read_viewperiod, wp_file, wp_start, wp_end
;
;  Read in viewperiods for the non-NOAA stations.
;
others = ['Bochum', 'Kiel', 'Koganei', 'KSWC']
delvarx, other_start, other_end, other_name
for iother = 0,n_elements(others)-1 do begin
    file = concat_dir(ssc_views, 'ahead_' + strlowcase(others[iother]) + $
                      '.view')
    if not file_exist(file) then begin
        message = 'File ' + file + ' not found'
        goto, handle_error
    endif
;
    ssc_read_viewperiod, file, vstart, vend
    if n_elements(other_start) eq 0 then begin
        other_start = vstart
        other_end   = vend
        other_name  = replicate(others[iother], n_elements(vend))
    end else begin
        other_start = [other_start, vstart]
        other_end   = [other_end,   vend]
        other_name  = [other_name,  replicate(others[iother], n_elements(vend))]
    endelse
endfor
;
;  Determine the date range to process.
;
fb_mjd = min((tai2utc(fb_start)).mjd)
wp_mjd = min((tai2utc(wp_start)).mjd)
dsn_mjd = min((tai2utc(dsn_start)).mjd)
mjd0 = (fb_mjd < wp_mjd) > dsn_mjd
;
fb_mjd = max((tai2utc(fb_end)).mjd)
wp_mjd = max((tai2utc(wp_end)).mjd)
dsn_mjd = max((tai2utc(dsn_end)).mjd)
mjd1 = (fb_mjd > wp_mjd) < dsn_mjd
;
;  Use the Z buffer for the plots.
;
dname = !d.name
setplot, 'Z'
device, set_resolution=[1000,480]
linecolor, 1, 'green'
linecolor, 2, 'yellow'
linecolor, 3, 'red'
ytickname = [' ', 'Wallops', ' ', 'Fairbanks', ' ']
;
;  Open the output file.
;
openw, outlun, concat_dir(ssc_noaa, 'noaa_views.csv'), /get_lun
;
;  Step through the days
;
for mjd = mjd0,mjd1 do begin
    utc0 = {mjd: mjd, time: 0L}
    utc1 = {mjd: mjd+1, time: 0L}
    date = utc2str(utc0, /date_only)
    sdate = anytim2cal(utc0, form=8, /date)
    utplot, [utc0, utc1], [0,2], /nodata, xstyle=1, title=date, xtitle='Time', $
            ytickname=ytickname, _extra=_extra
    xpos = !x.crange[0] + 0.05 * (!x.crange[1]-!x.crange[0])
    tai0 = utc2tai(utc0)
    tai1 = utc2tai(utc1)
;
;  Step through the NOAA stations.
;
    noaa = ['Fairbanks', 'Wallops']
    for inoaa=0,n_elements(noaa)-1 do begin
        case inoaa of
            0: begin
                xx_start = fb_start
                xx_end = fb_end
                ypos = 1.5
            end
            1: begin
                xx_start = wp_start
                xx_end = wp_end
                ypos = 0.5
            end
        endcase
;
;  Overplot the NOAA viewperiods.
;
        for ixx=0,n_elements(xx_start)-1 do begin
            xx0 = xx_start[ixx]
            xx1 = xx_end[ixx]
            if (xx0 lt tai1) and (xx1 gt tai0) then begin
                outplot, tai2utc([xx0,xx1]), [ypos, ypos], color=1, $
                         _extra=_extra
                printf, outlun, noaa[inoaa] + ',' + tai2utc(xx0,/ccsds) + $
                        ',' + tai2utc(xx1,/ccsds)
                if xx0 gt tai0 then begin
                    utc = tai2utc(xx0)
                    time = utc2str(utc, /time_only, /truncate)
                    xyouts, utc.time/1000., ypos, '  ' + time, color=1, $
                            alignment=0, orientation=80, _extra=_extra
                endif
                if xx1 lt tai1 then begin
                    utc = tai2utc(xx1)
                    time = utc2str(utc, /time_only, /truncate)
                    xyouts, utc.time/1000., ypos, '  ' + time, color=1, $
                            alignment=0, orientation=80, _extra=_extra
                endif
;
                for iother=0,n_elements(other_start)-1 do begin
                    istart = other_start[iother]
                    iend   = other_end[iother]
                    if (istart lt xx1) and (iend gt xx0) then begin
                        outplot, tai2utc([istart>xx0, iend<xx1]), $
                                 [ypos, ypos], color=2, _extra=_extra
                        printf, outlun, other_name[iother] + ',' + $
                                tai2utc(istart,/ccsds) + ',' + $
                                tai2utc(iend,/ccsds)
                    endif
                endfor
;
                for idsn=0,n_elements(dsn_start)-1 do begin
                    istart = dsn_start[idsn]
                    iend   = dsn_end[idsn]
                    if (istart lt xx1) and (iend gt xx0) then begin
                        outplot, tai2utc([istart>xx0, iend<xx1]), $
                                 [ypos, ypos], color=3, _extra=_extra
                        printf, outlun, 'DSN,' + tai2utc(istart,/ccsds) + $
                                ',' + tai2utc(iend,/ccsds)
                        if istart gt xx0 then begin
                            utc = tai2utc(istart)
                            time = utc2str(utc, /time_only, /truncate)
                            xyouts, utc.time/1000., ypos, '  ' + time, $
                                    color=3, alignment=0, orientation=-80, $
                                    _extra=_extra
                        endif
                        if iend lt xx1 then begin
                            utc = tai2utc(iend)
                            time = utc2str(utc, /time_only, /truncate)
                            xyouts, utc.time/1000., ypos, '  ' + time, $
                                    color=3, alignment=0, orientation=-80, $
                                    _extra=_extra
                        endif
                    endif
                endfor          ;ISDN
            endif               ;Plotting viewperiod
        endfor                  ;IXX
        printf, outlun
    endfor                      ;INOAA
;
;  Read from Z buffer and write to file.
;
    img = tvread(r,g,b)
    outfile = concat_dir(ssc_noaa, 'noaa_' + sdate + '.gif')
    write_gif, outfile, img, r, g, b
endfor                          ;MJD
;
;  Remove old plots.
;
files = file_search(concat_dir(ssc_noaa, 'noaa_*.gif'), count=nfiles)
now = systime(1)
for ifile = 0,nfiles-1 do begin
    age = now - (file_info(files[ifile])).mtime
    if age gt 86400 then file_delete, files[ifile], /verbose
endfor
;
goto, exit_point
;
;  Error handling point.
;
handle_error:
if ssc_notify ne '' then mail, users=ssc_notify, $
                               subj='Problem parsing NOAA views', message
message, message, /continue
;
;  Exit point.
;
exit_point:
if n_elements(dname)  eq 1 then setplot, dname
if n_elements(outlun) eq 1 then free_lun, outlun
end
