;+
; Project     :	STEREO - SSC
;
; Name        :	SSC_WRITE_IIRV
;
; Purpose     :	Write out IIRV ephemeris files.
;
; Category    :	STEREO, Operations, Orbit
;
; Explanation :	This procedure writes out a daily ephemeris for one of the two
;               STEREO spacecraft in Improved Inter-Range Vector (IIRV)
;               format.  This format is used by some of the NOAA ground
;               stations to track the spacecraft between DSN passes.
;
; Syntax      :	SSC_WRITE_IIRV, SPACECRAFT, DATE
;
; Inputs      :	SPACECRAFT = Can be one of the following forms:
;
;                               'A'             'B'
;                               'STA'           'STB'
;                               'Ahead'         'Behind'
;                               'STEREO Ahead'  'STEREO Behind'
;                               'STEREO-Ahead'  'STEREO-Behind'
;                               'STEREO_Ahead'  'STEREO_Behind'
;
;                            Case is not important.  The NAIF numeric codes of
;                            -234 and -235 respectively can also be used.
;
;               DATE    = The date that the ephemeris will cover.  Can be in
;                         any format recognized by ANYTIM2UTC.  Only the date
;                         is used--any time information is ignored.
;
; Opt. Inputs :	None.
;
; Outputs     :	The ephemeris file is written to disk with a filename generated
;               from the spacecraft and date, e.g.
;
;                       ahead_20061113_01.iirv
;
;               The last two digits in the filename is a version number which
;               is incremented each time the procedure is called.  If the new
;               version is identical to the previous version, it is deleted.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG = If defined and passed, then any error messages will be
;                        returned to the user in this parameter rather than
;                        depending on the MESSAGE routine in IDL.  If no errors
;                        are encountered, then a null string is returned.  In
;                        order to use this feature, ERRMSG must be defined
;                        first, e.g.
;
;                               ERRMSG = ''
;                               SSC_WRITE_IIRV, ERRMSG=ERRMSG, ...
;                               IF ERRMSG NE '' THEN ...
;
;               Will also accept any LOAD_STEREO_SPICE keywords.
;
; Env. vars.  : SSC_STATIONS_IIRV_W = Directory to write IIRV files to.
;
; Calls       :	SSC_WRITE_IIRV_LINE, ANYTIM2UTC, ANYTIM2TAI, PARSE_STEREO_NAME,
;               LOAD_STEREO_SPICE, ANYTIM2CAL, CONCAT_DIR, TAI2UTC,
;               GET_STEREO_COORD, SSC_IIRV_CHECKSUM
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	The SPICE kernels will be loaded, if not loaded already.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 13-Nov-2006, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
;------------------------------------------------------------------------------
;
pro ssc_write_iirv_line, unit, line, nofeed=nofeed
writeu, unit, byte(line)
if not keyword_set(nofeed) then writeu, unit, [13b,13b,10b,10b]
end
;
;------------------------------------------------------------------------------
;
pro ssc_iirv_checksum, line
b = byte(line)
chksum = 0
for i=0,n_elements(b)-1 do begin
    sb = string(b[i])
    if valid_num(sb) then chksum = chksum + fix(sb)
    if sb eq '-' then chksum = chksum + 1
endfor
line = line + string(chksum, format='(I3.3)')
end
;
;------------------------------------------------------------------------------
;
pro ssc_write_iirv, spacecraft, date, errmsg=errmsg, _extra=_extra
on_error, 2
;
;  Determine the start and end times, based on the beginning and end of a
;  calendar day.
;
message = ''
utc = anytim2utc(date, errmsg=message)
if message ne '' then goto, handle_error
;
utc.time = 0
t0 = anytim2tai(utc, /nocorrect)
delta = 900.d0                  ;Fifteen minutes
nt = 96                         ;File will contain one additional entry
;
;  Determine which spacecraft was requested, and translate it into the proper
;  input for SPICE.  Determine the Support Identification Code and Vehicle
;  Identification Code combination.
;
sc = parse_stereo_name(spacecraft, ['ahead','behind'])
case sc of
    'ahead':  sic = '023401'
    'behind': sic = '023501'
    else: begin
        message = 'Unable to recognize spacecraft ' + strtrim(sc,2)
        goto, handle_error
    end
endcase
;
;  Make sure that the SPICE kernels are loaded.
;
load_stereo_spice, errmsg=message, _extra=_extra
if message ne '' then goto, handle_error
;
;  Create the filename based on the spacecraft and the date.
;
filename = sc + '_' + anytim2cal(utc, form=8, /date)
filename = concat_dir(getenv('SSC_STATIONS_IIRV_W'), filename)
;
;  Find any existing files, extract the highest index number, and increment it
;  by 1.
;
files = filename + '_??.iirv'
files = file_search( files, count=count)
if count gt 0 then begin
    s = sort(files)
    files = files(reverse(s))
    lastfile = files[0]
    underscore = strpos(lastfile, '_', /reverse_search)
    index = fix(strmid(lastfile, underscore+1, 2)) + 1
end else begin
    index = 1
    lastfile = ''
endelse
filename = filename + '_' + string(index,format='(I2.2)') + '.iirv'
;
;  Open the output file, and write out the header.
;
openw, unit, filename, /get_lun
id = 1234567
line = '03' + string(id,format='(I7.7)') + '010'
ssc_write_iirv_line, unit, line, /nofeed
;
;  Step through the times.
;
for i=0L,nt do begin
    t = t0 + delta * double(i)
    utc = tai2utc(t,/nocorrect,/external)
;
;  Get the position of the spacecraft in geocentric coordinates.
;
    state = get_stereo_coord(utc, sc, system='GEO')
;
;  Write out the line signaling the start of a record.
;
    ssc_write_iirv_line, unit, 'GIIRV MANY'
;
;  Write out the line containing the scid and time range
;
    line = '1111' + sic + string((i+1) mod 1000,format='(I3.3)') + $
      string(utc2doy(utc),    format='(I3.3)') + $
      string(utc.hour,        format='(I2.2)') + $
      string(utc.minute,      format='(I2.2)') + $
      string(utc.second,      format='(I2.2)') + $
      string(utc.millisecond, format='(I3.3)')
    ssc_iirv_checksum, line
    ssc_write_iirv_line, unit, line
;
;  Write out the line containing the position information.
;
    pos = string(round(state[0:2] * 1d3, /l64), format='(I13.12)')
    line = pos[0] + pos[1] + pos[2]
    ssc_iirv_checksum, line
    ssc_write_iirv_line, unit, line
;
;  Write out the line containing the velocity information.
;
    vel = string(round(state[3:5] * 1d6, /l64), format='(I13.12)')
    line = vel[0] + vel[1] + vel[2]
    ssc_iirv_checksum, line
    ssc_write_iirv_line, unit, line
;
;  Write out the line describing the satellite.
;
    line = '00000000000000000 0000000'
    ssc_iirv_checksum, line
    ssc_write_iirv_line, unit, line
;
;  Write out the end line.
;
    ssc_write_iirv_line, unit, 'ITERM GAQD'    
endfor
;
;  Close the output file and check to see if the file is identical to the
;  previously generated file, if any.
;
free_lun, unit
if index gt 1 then begin
    openr, unit1, /get_lun, filename
    openr, unit2, /get_lun, lastfile
    test = 0
    line1 = 'string'
    line2 = 'string'
    while (not eof(unit1)) and (not eof(unit2)) do begin
        readf, unit1, line1
        readf, unit2, line2
        test = test or (line1 ne line2)
    endwhile
    test = test or (not eof(unit1))
    test = test or (not eof(unit2))
    free_lun, unit1, unit2
    if not test then begin
        print, 'Deleting '+filename
        file_delete, filename
    endif
endif
;
return
;
;  Error handling point.
;
handle_error:
if n_elements(errmsg) eq 0 then message, message else $
  errmsg = 'ssc_write_iirv: ' + message
;
end
