;+
; Project     :	STEREO - SSC
;
; Name        :	SSC_WRITE_INP_AZEL
;
; Purpose     :	Write out INP ephemeris files in AZ/EL for specific stations
;
; Category    :	STEREO, Operations, Orbit
;
; Explanation : This procedure writes out INP ephemeris files for one of the
;               two STEREO spacecraft specific for a specific ground station.
;               This version differs from ssc_write_inp.pro in that Azimuth and
;               Elevation angles are written instead of X30, Y30.
;
; Syntax      :	SSC_WRITE_INP_AZEL, SPACECRAFT, STATION  [, DELTA=DELTA ]
;
; Inputs      :	SPACECRAFT = Can be one of the following forms:
;
;                               'A'             'B'
;                               'STA'           'STB'
;                               'Ahead'         'Behind'
;                               'STEREO Ahead'  'STEREO Behind'
;                               'STEREO-Ahead'  'STEREO-Behind'
;                               'STEREO_Ahead'  'STEREO_Behind'
;
;                            Case is not important.  The NAIF numeric codes of
;                            -234 and -235 respectively can also be used.
;
;               STATION = The name of the ground station, e.g. "wallops".
;
; Opt. Inputs :	None.
;
; Outputs     :	The ephemeris file is written to disk with a filename generated
;               from the spacecraft, date, and hour, e.g.
;
;                       ahead_20220623_16_inp_azel_wallops_01.txt
;
;               The last two digits in the filename is a version number which
;               is incremented each time the procedure is called.  If the new
;               version is identical to the previous version, it is deleted.
;
; Opt. Outputs:	None.
;
; Keywords    :	DELTA  = Step size in seconds.  Default is 600 (10 minutes).
;
;               ERRMSG = If defined and passed, then any error messages will be
;                        returned to the user in this parameter rather than
;                        depending on the MESSAGE routine in IDL.  If no errors
;                        are encountered, then a null string is returned.  In
;                        order to use this feature, ERRMSG must be defined
;                        first, e.g.
;
;                               ERRMSG = ''
;                               SSC_WRITE_INP_AZEL, ERRMSG=ERRMSG, ...
;                               IF ERRMSG NE '' THEN ...
;
; Env. vars.  : SSC_STATIONS_W = Directory to write station files to.  Each
;               station will have a subdirectory under this main directory.
;
; Calls       :	ANYTIM2UTC, PARSE_STEREO_NAME, LOAD_STEREO_SPICE_EARTH,
;               ANYTIM2CAL, CONCAT_DIR, GET_UTC, ANYTIM2TAI, TAI2UTC,
;               CSPICE_FURNSH, CSPICE_BODVAR, CSPICE_GEOREC, CSPICE_STR2ET,
;               CSPICE_SPKEZR, CSPICE_SXFORM, CSPICE_RECLAT, CSPICE_UNLOAD,
;               DIR_EXIST, MKDIR, LOAD_SUNSPICE, DElVARX, BOOST_ARRAY, UTC2STR
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	The SPICE kernels will be loaded, if not loaded already.
;
; Prev. Hist. :	Based on ssc_write_inp.pro
;
; History     :	Version 1, 19-Jul-2022, William Thompson, GSFC
;               Version 2, 02-Nov-2023, WTT, make azimuth positive 0-360
;
; Contact     :	WTHOMPSON
;-
;
pro ssc_write_inp_azel, spacecraft, station, delta=delta, _extra=_extra
on_error, 0
;
formfeed = string(12b)
creturn  = string(13b)
;
;  Make sure that delta is defined.
;
if n_elements(delta) ne 1 then delta = 600.d0
;
;  Determine which spacecraft was requested, and translate it into the proper
;  input for SPICE.  Determine the SPACEWARN object ID number.
;
sc = parse_stereo_name(spacecraft, ['STEREO AHEAD','STEREO BEHIND'])
case sc of
    'STEREO AHEAD': begin
        fname = 'ahead'
        sic = '234'
    end
    'STEREO BEHIND': begin
        fname = 'behind'
        sic = '235'
    end
    else: begin
        message = 'Unable to recognize spacecraft ' + strtrim(sc,2)
        goto, handle_error
    end
endcase
;
;  Open the viewperiods file, and read in the viewperiods.
;
path = concat_dir(getenv('SSC_STATIONS_W'), 'viewperiods')
if not dir_exist(path) then begin
    message = 'Viewperiods directory does not exist'
    goto, handle_error
endif
;
viewfile = concat_dir(path, fname + '_' + strlowcase(station) + '.view')
if not file_exist(viewfile) then begin
    message = 'Viewperiods file ' + viewfile + ' does not exist'
    goto, handle_error
endif
;
catch, error_status
if error_status ne 0 then begin
    message = !error_state.msg
    catch, /cancel
    goto, handle_error
endif
openr, unit, viewfile, /get_lun
;
line = 'string'
for iline=0,7 do readf, unit, line  ;Read past header
;
delvarx, rise, set
while not eof(unit) do begin
    readf, unit, line
    words = strsplit(line, /extract)
    boost_array, rise, words[0]
    boost_array, set,  words[1]
endwhile
free_lun, unit
catch, /cancel
;
;  Make sure that the SPICE kernels are loaded.
;
message = ''
load_sunspice, sc, errmsg=message, _extra=_extra
if message ne '' then goto, handle_error
load_stereo_spice_earth, errmsg=message, _extra=_extra
if message ne '' then goto, handle_error
;
;  Load the station frame file.
;
datpath = concat_dir(getenv('SSW_SSC'), 'data/spice/stations')
tf_file = concat_dir(datpath, strlowcase(station) + '.tf')
cspice_furnsh, tf_file
;
;  Make sure that the station directory exists.
;
path = concat_dir(getenv('SSC_STATIONS_W'), strlowcase(station))
if not dir_exist(path) then mk_dir, path
;
;  Get the origin from the stations.dat file.
;
openr, in, concat_dir(datpath, 'stations.dat'), /get_lun
line = 'string'
done = 0
while not (done or eof(in)) do begin
    readf, in, line
    line = strcompress(strtrim(line,2))
    words = strsplit(line, ' ', /extract)
    if words[0] eq strlowcase(station) then begin
        lat_r = double(words[1]) * !dpi / 180.d0
        lon_r = double(words[2]) * !dpi / 180.d0
        alt   = double(words[3]) / 1000.d0 ;Convert meters -> kilometers
        done = 1
    endif
endwhile
free_lun, in
if not done then begin
    message = 'Station ' + station + ' not found in stations.dat'
    goto, handle_error
endif
;
cspice_bodvar, 399, 'RADII' , radii
flat = (radii[0]-radii[2]) / radii[0]
cspice_georec, lon_r, lat_r, alt, radii[0], flat, origin
origin = [origin, 0, 0, 0]
;
;  Step through the viewperiods
;
file_open = 0
nview = n_elements(rise)
for iview=0,nview-1 do begin
    start_time = rise[iview]
    end_time = set[iview]
;
;  Create the filename based on the spacecraft and the date.
;
    if not file_open then begin
        message = ''
        sdate = anytim2cal(start_time, form=8, /date, errmsg=message)
        if message ne '' then goto, handle_error
        ext = anytim2utc(start_time, /external)
        sdate = sdate + '_' + string(ext.hour, format='(I2.2)')
        filename = fname + '_' + sdate + '_inp_azel_' + strlowcase(station)
        filename = concat_dir(path, filename)
;
;  Find any existing files, extract the highest index number, and increment it
;  by 1.
;
        files = filename + '_??.txt'
        files = file_search(files, count=count)
        if count gt 0 then begin
            s = sort(files)
            files = files(reverse(s))
            lastfile = files[0]
            underscore = strpos(lastfile, '_', /reverse_search)
            index = fix(strmid(lastfile, underscore+1, 2)) + 1
        end else begin
            index = 1
            lastfile = ''
        endelse
        filename = filename + '_' + string(index,format='(I2.2)') + '.txt'
;
;  Format the current date/time.
;
        get_utc, utc, /ext
        s_now = strmid(ntrim(utc.year), 2, 2) + '/' + $
                string(utc2doy(utc), format='(I3.3)') + '/' + $
                utc2str(utc, /time_only)
    endif
;
;  Format the start and end times.
;
    utc = anytim2utc(start_time, /ext)
    aos = strmid(ntrim(utc.year), 2, 2) + '/' + $
          string(utc2doy(utc), format='(I3.3)') + '/' + $
          utc2str(utc, /time_only, /truncate)
;
    utc = anytim2utc(end_time, /ext)
    los = strmid(ntrim(utc.year), 2, 2) + '/' + $
          string(utc2doy(utc), format='(I3.3)') + '/' + $
          utc2str(utc, /time_only, /truncate)
;
;  Formulate the header
;
    header = '     ' + formfeed
    header = [header, 'Acquisition Data Generated List Report        ' + $
              s_now + '       PAGE ']
    header = [header, 'USER ID: naas               DB ID: STEREO-' + $
              strupcase(strmid(fname,0,1)) + '             DB: NAAS']
    header = [header, '']
    header = [header, '']
    header = [header, '']
    header = [header, 'PRODUCT ID:00000001      SIC/VIC: 0' + sic + $
              '/01       Generated Time: ' + s_now]
    header = [header, 'SET NO.:         1       MSG. TYPE:  INP' + $
              '       SET TYPE:       ONORBIT']
    header = [header, 'Routers: GDRO']
    header = [header, '']
    header = [header, 'Site:   WL53']
    header = [header, '                 AOS   ' + aos + '          LOS   ' + $
              los]
    header = [header, '']
    header = [header, 'Comment:  ']
    header = [header, '']
    header = [header, '             AZ      El']
    header = [header, '   GMT      (DEG)   (DEG)']
    header = [header, '']
;
;  If not already open, then open the output file.
;
    if not file_open then begin
        openw, unit, filename, /get_lun
        file_open = 1
        ipage = 1
    endif
;
;  Write out the header.
;
    printf, unit, header[0] + creturn
    printf, unit, header[1] + string(ipage, format='(I3)') + creturn
    for iheader=2,n_elements(header)-1 do $
      printf, unit, header[iheader] + creturn
    ipage = ipage + 1
;
;  Step through the times.
;
    t0 = anytim2tai(start_time, /nocorrect)
    t1 = anytim2tai(end_time, /nocorrect)
;
;  If necessary, move to the next page.  Change the header.
;
    iout = n_elements(header)
    for t=t0,t1,delta do begin
        if iout eq 38 then begin
            for i=1,6 do printf, unit, ' ' + creturn
            if n_elements(header) eq 18 then $
              header = [header[0:4], header[11:12], header[15:17]]
            printf, unit, header[0] + creturn
            printf, unit, header[1] + string(ipage, format='(I3)') + creturn
            for iheader=2,n_elements(header)-1 do $
              printf, unit, header[iheader] + creturn
            iout = n_elements(header)
            ipage = ipage + 1
        endif
;
        utc = tai2utc(t,/nocorrect,/ccsds)
        cspice_str2et, utc, et
;
;  Get the position of the spacecraft in geocentric coordinates, and subtract
;  the origin.  Calculate the component of the velocity along the radial
;  vector.
;
        cspice_spkezr, sc, et, 'ITRF93', 'lt+s', 'Earth', state, ltime
        state = state - origin
        velocity = total(state[0:2]*state[3:5]/sqrt(total(state[0:2]^2)))
;
;  Convert to the station frame.
;
        cspice_sxform, 'ITRF93', strupcase(station) + '_TOPO', et, xform
        state = transpose(xform) # state
;
;  Convert into azimuth and elevation.
;
        cspice_reclat, state[0:2], distance, azimuth, elevation
        azimuth   = -azimuth  * (180.d0 / !dpi)
        elevation = elevation * (180.d0 / !dpi)
        if azimuth lt 0 then azimuth = azimuth + 360
        time = anytim2utc(utc, /ccsds, /time_only, /truncate)
        printf, unit, format='(1x,A8,2F8.2,A1)', time, azimuth, elevation, $
                creturn
        iout = iout + 1
    endfor
;
;  Determine whether to continue with the present file, or open a new one.  
;
    close = 0
    if iview eq (nview-1) then close = 1 else $
      if (anytim2tai(rise[iview+1]) - anytim2tai(end_time)) gt 14400 then $
        close = 1
;
;  Close the output file and check to see if the file is identical to the
;  previously generated file, if any.
;
    if close then begin
        free_lun, unit
        file_open = 0
        if index gt 1 then begin
            openr, unit1, /get_lun, filename
            openr, unit2, /get_lun, lastfile
            test = 0
            line1 = 'string'
            line2 = 'string'
;
;  Skip over any lines containing s_now.
;
            while (not eof(unit1)) and (not eof(unit2)) do begin
                readf, unit1, line1
                readf, unit2, line2
                if strpos(line1, s_now) lt 0 then $
                  test = test or (line1 ne line2)
            endwhile
            test = test or (not eof(unit1))
            test = test or (not eof(unit2))
            free_lun, unit1, unit2
            if not test then begin
                print, 'Deleting '+filename
                file_delete, filename
            end else print, 'Wrote ' + filename
        end else print, 'Wrote ' + filename
;
;  Otherwise, write out enough blank lines to put the next header in the
;  proper position.
;
    end else for jout=iout+1,44 do printf, unit, ' ' + creturn
endfor                          ;viewperiods
;
cspice_unload, tf_file
return
;
;  Error handling point.
;
handle_error:
if n_elements(errmsg) eq 0 then message, message else $
  errmsg = 'ssc_write_inp_azel: ' + message
;
end
