;+
; Project     :	STEREO - SSC
;
; Name        :	STEREO_GEOGR_DEMO
;
; Purpose     :	Geographical STEREO SPICE kernel demonstration
;
; Category    :	STEREO, Orbit
;
; Explanation :	This procedure demonstrates the process of deriving the
;               position of the two STEREO spacecraft in geographic
;               coordinates.  The appropriate SPICE kernels are loaded, the
;               date/time is converted to the ephemeris time which SPICE uses,
;               and the spacecraft letter identifier is converted to the
;               appropriate code.  CSPICE_SPKEZR, CSPICE_SUBPT and
;               CSPICE_RECLAT are used to derive the position and altitude.
;               Finally, at the end of the procedure, the kernels are unloaded.
;
;               It is not intended that this procedure be used directly.
;               Instead, it is a simple demo of the process of using STEREO
;               SPICE kernels.
;
; Syntax      :	STEREO_GEOGR_DEMO, DATE, SPACECRAFT, LON, LAT, ALT
;
; Examples    :	stereo_geogr_demo, '2006-05-06T11:30:00', 'A', lon, lat, alt
;
; Inputs      :	DATE       = The date and time.  This can be input in any
;                            format accepted by ANYTIM2UTC.
;               SPACECRAFT = Either "A" or "B".
;
; Opt. Inputs :	None.
;
; Outputs     :	LON = The geographic longitude, in degrees.
;               LAT = The geographic latitude, in degrees.
;               ALT = The altitude above the surface, in kilometers.
;
; Opt. Outputs:	None.
;
; Keywords    :	CORR = Aberration correction.  Default is 'None'.  Other
;                      possible values are:
;                       'LT'    Light travel time
;                       'LT+S'  Light travel time plus stellar aberration
;
; Calls       :	ANYTIM2UTC, CONCAT_DIR, CSPICE_FURNSH, CSPICE_SUBPT,
;               CSPICE_RECLAT, CSPICE_UNLOAD
;
; Common      :	None.
;
; Restrictions:	This procedure works in conjunction with the Icy/CSPICE
;               package, which is implemented as an IDL Dynamically Loadable
;               Module (DLM).  The Icy source code can be downloaded from
;
;                       ftp://naif.jpl.nasa.gov/pub/naif/toolkit/IDL
;
; Side effects:	None.
;
; Prev. Hist. :	Based on subpt.pro from the Icy distribution.
;
; History     :	Version 1, W. Thompson, 12 May 2004
;
; Contact     :	WTHOMPSON
;-
;
pro stereo_geogr_demo, date, spacecraft, lon, lat, alt, corr=k_corr
;
on_error, 2
if n_params() ne 5 then message, $
        'Syntax:  STEREO_GEOGR_DEMO, DATE, SPACECRAFT, LON, LAT, ALT'
;
;  Determine which spacecraft was requested, and translate it into the proper
;  input for SPICE.  Some SPICE routines use an ID number, which is positive
;  for planets and other solar system bodies, and negative for spacecraft.
;  Other routines use a character string name.  A few routines which take
;  character strings, such as SPKEZR but not SUBPT, will also accept the ID
;  number formatted as a string.
;
case strupcase(spacecraft) of
    'A': sc = 'STEREO AHEAD'
    'B': sc = 'STEREO BEHIND'
    else: message, 'Unrecognized spacecraft'
endcase
;
;  Convert the date/time to UTC.
;
errmsg = ''
utc = anytim2utc(date,/ccsds,errmsg=errmsg)
if errmsg ne '' then message, errmsg
;
;  Parse the keywords.
;
if n_elements(k_corr) eq 1 then corr = k_corr else corr = 'None'
;
;  Define the path to the demo files.
;
datapath = concat_dir('$SSW', 'stereo', /DIR)
datapath = concat_dir(datapath, 'ssc', /DIR)
datapath = concat_dir(datapath, 'data', /DIR)
datapath = concat_dir(datapath, 'spice_demo', /DIR)
;
;  Define the files which need to be loaded.  The STEREO ephemeris file is
;  preliminary, based on the projected launch date.  To support reading this
;  file, two other files must also be loaded.  The file 'naif0007.tls' contains
;  leap second information used to support the conversion from UTC to the
;  ephemeris time (ET) used by SPICE.  The planetary orbit ephemeris file,
;  'de405.bsp', is the same file used in generating the STEREO ephemeris, and
;  is required to put the STEREO ephemeris into its proper context relative to
;  the solar system barycenter where SPICE does its calculations.  It also
;  allows STEREO positions to be calculated relative to various solar system
;  objects.  Solar and planetary physical data are stored in 'pck00007.tpc'.
;  These supporting files were downloaded from
;
;	ftp://naif.jpl.nasa.gov/pub/naif/generic_kernels
;
tm_file = concat_dir(datapath, 'naif0007.tls')          ;Leapseconds
de_file = concat_dir(datapath, 'de405.bsp')             ;Planetary orbits
pl_file = concat_dir(datapath, 'pck00007.tpc')          ;Planetary phys. data
st_file = concat_dir(datapath, 'stereo_20060211a_nominal.bsp')
;
;  Load the files
;
cspice_furnsh, tm_file
cspice_furnsh, de_file
cspice_furnsh, pl_file
cspice_furnsh, st_file
;
;  Convert the date/time to ephemeris time.
;
cspice_str2et, utc, et
;
;  Get the planetocentric position.
;
cspice_spkezr, sc, et, 'IAU_Earth', corr, 'Earth', state, ltime
;
;  Get the planetary radii data, and convert to planetographic latitude and
;  longitude.
;
cspice_bodvar, 399, 'RADII', radii
flat = (radii[0]-radii[2]) / radii[0]
cspice_recgeo, state[0:2], radii[0], flat, lon, lat, alt
lon = lon * 180d / !dpi
lat = lat * 180d / !dpi
;
;  Unload the files
;
cspice_unload, st_file
cspice_unload, pl_file
cspice_unload, de_file
cspice_unload, tm_file
;
end
