;+
; Project     :	STEREO - SSC
;
; Name        :	SSC_PURGE_TELEM
;
; Purpose     :	Purge old telemetry files.
;
; Category    :	STEREO, telemetry
;
; Explanation :	This routine goes through the directories of level-0 telemetry
;               files, and deletes any .ptp files which have been superceded by
;               .fin files.
;
; Syntax      :	ssc_purge_telem
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Env. Vars.  : STEREO_TELEM = The top of the telemetry file tree.
;
; Calls       :	CONCAT_DIR, BREAK_FILE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 07-Feb-2007, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
pro ssc_purge_telem
;
sc = ['ahead','behind']
ins = ['cdh','gc','impact','plastc','secchi','swaves']
;
;  Get the current time, and define 
;
now = systime(1)
six_months = 15552000
;
stereo_telem = getenv('STEREO_TELEM')
;
;  Step through the spacecraft.
;
for isc=0,1 do begin
    path_sc = concat_dir(stereo_telem, sc[isc])
;
;  Step through the instruments.
;
    for iins = 0,5 do begin
        path_ins = concat_dir(path_sc, ins[iins])
;
;  Step through the year directories.
;
        years = file_search(concat_dir(path_ins,'*'), count=nyears, $
                            /test_directory)
        if nyears gt 0 then for iyear=0,nyears-1 do begin
            break_file, years[iyear], disk, dir, year
;
;  Step through the month directories.
;
            months = file_search(concat_dir(years[iyear],'*'), count=nmonths, $
                            /test_directory)
            if nmonths gt 0 then for imonth=0,nmonths-1 do begin
;
;  Find all properly named files, and get the file sizes and modification time.
;
                testname = ins[iins] + '_' + sc[isc] + '_' + $
                  string(year, format='(I4.4)') + '_???_?_??.???'
                file = file_search(concat_dir(months[imonth], testname), $
                                   count=nfiles)
                if nfiles gt 0 then begin
                    info = file_info(file)
                    mtime = info.mtime
                    fsize = info.size
;
;  Extract the day-of-year, day part, file version, and extension.
;
                    break_file, file, disk, dir, name, ext
                    fields = strsplit(name[0], '_')
                    doy  = strmid(name, fields[3], 3)
                    part = strmid(name, fields[4], 1)
                    vers = strmid(name, fields[5], 2)
;
;  Step through each day-of-year.
;
                    all_doys = all_vals(doy)
                    for idoy=0,n_elements(all_doys)-1 do begin
                        wd = where(doy eq all_doys[idoy])
;
;  Step through each day part (SECCHI only).
;
                        all_parts = all_vals(part[wd])
                        for ipart=0,n_elements(all_parts)-1 do begin
                            wp = wd[where(part[wd] eq all_parts[ipart])]
;
;  Make sure that there are .fin files, and get the maximum version number,
;  size, and modification time.
;
                            wfin = where(ext[wp] eq '.fin', nfin)
                            if nfin gt 0 then begin
                                wfin = wp[wfin]
                                vers0  = max( vers[wfin])
                                mtime0 = max(mtime[wfin])
                                fsize0 = max(fsize[wfin])
;
;  Delete any .ptp files which are older than the .fin files, and are not
;  larger.  Only delete files which are at least six months old.
;
                                wptp = where(ext[wp] eq '.ptp', nptp)
                                if nptp gt 0 then begin
                                    wptp = wp[wptp]
                                    for iptp=0,nptp-1 do begin
                                        mtime1 = mtime[wptp[iptp]]
                                        fsize1 = fsize[wptp[iptp]]
                                        if (mtime1 lt mtime0) and $
                                          (fsize1 le fsize0) and $
                                          ((now-mtime1) gt six_months) $
                                          then begin
                                            print, file[wptp[iptp]]
                                            file_delete, file[wptp[iptp]]
                                        endif
                                    endfor
                                endif           ;ntpt gt 0
                            endif               ;nfin gt 0
;
                        endfor  ;ipart
                    endfor      ;idoy
                endif           ;nfiles gt 0
            endfor              ;imonth
        endfor                  ;iyear
    endfor                      ;iins
endfor                          ;isc
;
end
